% ------------------------------------------------------------ %
% PURPOSE:  This program reads in the .txt file containing the %
%	    data from Compustat. This will be a simply program %
%	    in that it will read in the file and save it as a  %
%	    matlab file for further analysis. The program has  %
%	    now been extended to also compute the moments of   %
%	    interest from the data. It will probably also      %
%	    include the computation of the covariance matrix   %
%	    of the moments using the influence function method %
%	    as in Toni's papers.  This could get messy.	       %
% ------------------------------------------------------------ %
% Written By:  Missaka W.				       %
% Date: 04/12/2007					       %
% Last Update:						       %
%       01/25/2011					       %
%       The program has been overhauled to have substantially  %
% more moments, and everything has been brought down to 1      %
% moment scales (means, stds, corrs).  Lets hope these moments %
% are somewhat reasonable with the data.    	       	       %
% ------------------------------------------------------------ %
function [] = readdata(indch)

addpath ../basic;

% Define the input and output files for us to read and write
if strcmp(indch, '')
    infile = 'maindata.txt';
    svfil  = 'moments.mat';
else
    infile = strcat(indch, 'data.txt');
    svfil = strcat('moments_', indch, '.mat');
end

% Time to read in the data
[gvkey, yeara, siz, rds, lrds, salek, sgrow, q, roa, lroa, invk, linvk, fcs] = ...
	textread(infile, '%f%f%f%f%f%f%f%f%f%f%f%f%f', 'emptyvalue', NaN, ...
			 'delimiter', '\t', 'headerlines', 1);

% Get the length of the data set
n = length(gvkey);
iota = ones(n, 1);

% Find out the observations with nonmissing values for lagged income
k = find(1 - isnan(lroa));
dk = zeros(n, 1); dk(k) = 1;

% Do a similar thing for lagged r&d
j = find(1 - isnan(lrds));
dj = zeros(n, 1); dj(j) = 1;

% Do the same for fixed costs
l = find(1 - isnan(fcs));
dl = zeros(n, 1); dl(l) = 1;

% We also need to run it for investment
m = find(1 - isnan(linvk));
dm = zeros(n, 1); dm(m) = 1;

% Now we can construct the moments of interest to match.  This will 
% include some first and second moments as well as one or two regression
% coefficients.

% First do r&d
fm_rd = mean(rds);
std_rd = std(rds, 1);
fm_lrd = mean(lrds(j));

% Lets compute the auto-regressive term for rd
rds_m = rds - mean(rds(j));
lrds_m = lrds - mean(lrds(j));
rho_rd = regress(rds_m(j), lrds_m(j));

% Here we do q by r&d type - matching 2nd moment of q will be tough
fm_q = mean(q);
std_q = std(q, 1);

% Lets do the cash flow moments as well
fm_roa = mean(roa);
std_roa = std(roa, 1);

% Now pick up investment
fm_invk = mean(invk);   % No longer a moment to be considered
std_invk = std(invk, 1);

% We also need the autoregression for investment
beta_inv = regress(invk(m), adc(linvk(m)));
rho_inv = beta_inv(2);

% We also want the sales to capital ratio
fm_sk = mean(salek);

% We need the mean of sales growth elsewhere
fm_sg = mean(sgrow(j));

% We also want the mean of fixed costs
fm_fs = mean(fcs(l));

% Finish up by picking up the auto-correlation term for cash flows
beta = regress(roa(k), adc(lroa(k)));
rho_roa = beta(2);

% Compute the residual of the above regressions
res_rd = rds_m - rho_rd*lrds_m; 
res_roa = roa - beta(1) - beta(2)*lroa;
res_inv = invk - beta_inv(1) - beta_inv(2)*linvk;

% We also want to pick up a bunch of correlations
cord_roa = corr(rds, roa);
cord_inv = corr(rds, invk);
cord_q = corr(rds, q);
coinv_q = corr(invk, q);
cosg_lrd = corr(lrds(j), sgrow(j));
cord_k = corr(rds, siz);

% Report mean sales growth values
disp 'Mean sales growth'
fm_sg

% Lets just display all the moments together
% Lets take out the covariances for now.
moms = [fm_rd std_rd fm_sk fm_fs fm_q fm_roa std_roa std_invk...
        rho_roa rho_rd rho_inv cord_inv cord_q cord_k coinv_q cosg_lrd];

% ------------------------------------------------------------ %
% Now that we have the moments, the next task is to compute their
% covariance matrix.  This requires some amount of care.
% ------------------------------------------------------------ %

% First step involves computing the residuals for the regressors
% and then transforming into the R function in EW.  This function
% corresponds to (y-xb)x.
% We have to do an adjustment for dealing with square roots here.
ri_fmrd = (rds - fm_rd);
ri_stdrd = (rds.^2 - fm_rd^2 - std_rd^2)/(2*std_rd);

res_rd(~dj) = 0;
lrds_m(~dj) = 0;
lroa(~dk) = 0;
linvk(~dm) = 0;
ri_rhord = lrds_m.*(res_rd);

ri_fmq = (q - fm_q);

ri_fmsk = (salek - fm_sk);

ri_fmfs = (fcs - fm_fs);
ri_fmfs(~dl) = 0;

ri_fmroa = (roa - fm_roa);
ri_stdroa = (roa.^2 - fm_roa^2 - std_roa^2)/(2*std_roa);

res_roa(~dk) = 0;
ri_rhoroa = lroa.*(res_roa);

res_inv(~dm) = 0;
ri_rhoinv = linvk.*res_inv;

ri_stdinvk = (invk.^2 - fm_invk^2 - std_invk^2)/(2*std_invk);

ri_cordinv = (invk.*rds - fm_rd*fm_invk)/(std_rd*std_invk) - cord_inv;
ri_cordq = (q.*rds - fm_q*fm_rd)/(std_rd*std_q) - cord_q;
ri_cordk = (siz.*rds - mean(siz)*fm_rd)/(std(siz, 1)*std_rd) - cord_k;
ri_coinvq = (q.*invk - fm_q*fm_invk)/(std_invk*std_q) - coinv_q;
ri_cosglrd = zeros(length(ri_fmq), 1);
ri_cosglrd(j) = (lrds(j).*sgrow(j) - fm_lrd*fm_sg)/...
	     (std(lrds(j), 1)*std(sgrow(j), 1)) - cosg_lrd;
ri_cosglrd(~j) = 0;

% Now we want to compute the Q matrices for each moment and then 
% derive the influence functions.  This will be done on a regressor
% by regressor basis.  The notation will be messy.

Q_iota = mean(iota.^2);
Q_lroa = mean(lroa.^2);
Q_linv = mean(linvk.^2);
Q_lrds = mean(lrds_m.^2);
Q_fcs = mean(dl.^2);

% Now we can construct the influence functions.  We will be doing
% this on a moment by moment basis.
phi_fmrd = inv(Q_iota)*ri_fmrd;
phi_stdrd = inv(Q_iota)*ri_stdrd;

phi_rhord = inv(Q_lrds)*ri_rhord;

phi_fmsk = inv(Q_iota)*ri_fmsk;
phi_fmq = inv(Q_iota)*ri_fmq;

phi_fmfs = inv(Q_fcs)*ri_fmfs;

phi_fmroa = inv(Q_iota)*ri_fmroa;
phi_stdroa = inv(Q_iota)*ri_stdroa;

phi_rhoroa = inv(Q_lroa)*ri_rhoroa;
phi_rhoinv = inv(Q_linv)*ri_rhoinv;

phi_stdinvk = inv(Q_iota)*ri_stdinvk;

phi_cordinv = inv(Q_iota)*ri_cordinv;
phi_cordq = inv(Q_iota)*ri_cordq;
phi_cordk = inv(Q_iota)*ri_cordk;
phi_coinvq = inv(Q_iota)*ri_coinvq;
phi_cosglrd = inv(Q_iota)*ri_cosglrd;

% Now we stack all the influence functions and take their inner
% product.  This will give us the asymptotic covariance matrix for
% the moments.
phimat = [phi_fmrd phi_stdrd phi_fmsk phi_fmfs phi_fmq phi_fmroa ...
          phi_stdroa phi_stdinvk phi_rhoroa phi_rhord phi_rhoinv ...
	  phi_cordinv phi_cordq phi_cordk phi_coinvq phi_cosglrd];

phi = (phimat'*phimat)/n;
sqrt(diag(phi)/n)

% Save the matrix containing the data on the moments.
save(svfil, 'moms', 'phi', 'n');
