% ------------------------------------------------------------ %
% PURPOSE: This program is written to evaluate the impact of   %
%	   exogeneous changes in some of the key parameters on %
%	   the simulated moments and other key parameters of   %
%	   the model. The main focus will be on the impact of  %
%	   these changes on the r&d sector.  Similar exercises %
%	   will be carried out for the extended models and the %
%          final results will be tabulated in a short tables.  %  
% ------------------------------------------------------------ %

clear;
tic;
addpath ../Model;
addpath ../Quad;
addpath ../Simul;
addpath ../RDMult;
addpath ../basic;
% warning('off', 'Matlab:log:logOfZero');

% Load in the parameters from the data;
load ../Data/moments;
load gmat;

data_vals{1} = moms;
data_vals{2} = n;
data_vals{3} = phi;
data_vals{4} = g;
data_vals{5} = 0.904;
data_vals{6} = r;

% Define the matrices to be used as inputs
% Set the parameters we do not optimize over
fc = 0.239;
delta = 0.165;
beta = 1/1.04;
srate = 0.025;         % This will be modified later
acc = 0.01;	       % Accuracy of fit

% Parameters about the function call to solve for the steady state
sim_ind = 51;  %51

% We also want to pick up tax data
trat_rd = 0.312;

% Panel stuff (keep it small for now)
nFirms = 5176; 
nYears = 100;  

% Put stuff together;
fix_paras = [fc beta delta sim_ind nFirms nYears trat_rd srate];

% Start with a guess for the initial value
theta = 0.79051;
nu = 0.35811;
rho = 0.80070; 
sigma = 0.42781;
lambda = 0.03465;
a = 9.26452;
b = 4.12264;
gamma = 0.18020;
mu = -0.30;
  
% Put it together
guess_p = [theta nu rho sigma lambda a b gamma];

% Here we define all the other stuff needed for SA to work
objfun = 'GMMObj_multmod';	       % GMM Objective function

global fixgrid
fixgrid = 0;

% This part calls the computations using the solved values
[fval, moms, ex_diags, grd] = feval(objfun, guess_p, fix_paras, data_vals);

%% Lets call it again with a new set of initial value functions
m = 0;
while abs(data_vals{5} - ex_diags(2)) > acc
      data_vals{4} = grd;
      data_vals{5} = (data_vals{5}+ex_diags(2))/2;
      [fval, moms, ex_diags, grd] = feval(objfun, guess_p, fix_paras, data_vals);
      m = m + 1
end

muk = ex_diags(3);

% Lets save the fix cost and pass it along the next time - overwrite this if we want to
% data_vals{8} = sfc;

fixgrid = 1;
% The next thing is to try and imagine what happens with changes to lambda
% Lets start with having a .10 drop in lambda
lfrac = .05;
lambda_d = lambda * (1+lfrac);

guess_pld = [theta nu rho sigma lambda_d a b gamma];

% Call the objective function again to see what pops out
[fvalld, momsld, ex_diagsld, grd] = feval(objfun, guess_pld, fix_paras, data_vals);

data_valsld = data_vals;
%% Lets call it again with a new set of initial value functions
m = 0;
while abs(data_valsld{5} - ex_diagsld(2)) > acc
      data_valsld{4} = grd;
      data_valsld{5} = (data_valsld{5}+ex_diagsld(2))/2;
      [fvalld, momsld, ex_diagsld, grd] = feval(objfun, guess_pld, fix_paras, data_valsld);
      m = m + 1
end

% We are going to call again after adjusting the fixed cost
kld_rat = ex_diagsld(3) / muk;
fc_ld = fc*kld_rat^theta;

% Now we see what the final result looks like with the adjusted
% fixed cost.
fix_parasld = fix_paras;
fix_parasld(1) = fc_ld;
[fvallf, momslf, ex_diagslf, grd] = feval(objfun, guess_pld, ...
                                          fix_parasld, data_valsld);

% Here we start tinkering with a subsidy for tax rates - start with 5%
sub_rate = 0.05;

% Lets update the parameter set 
fix_paras_sub = [fc beta delta sim_ind nFirms nYears trat_rd sub_rate];

% Redefine the data values
data_valssub = data_vals;

% Call the function to test things 
[fvalsub, momssub, ex_diagssub, grd] = feval(objfun, guess_p, fix_paras_sub, data_vals);

%% Lets call it again with a new set of initial value functions
m = 0;
while abs(data_valssub{5} - ex_diagssub(2)) > acc
      data_valssub{4} = grd;
      data_valssub{5} = (data_valssub{5}+ex_diagssub(2))/2;
      [fvalsub, momssub, ex_diagssub, grd] = feval(objfun, guess_p, fix_paras_sub, data_valssub);
      m = m + 1
end

% We are going to call again after adjusting the fixed cost
ksub_rat = ex_diagssub(3) / muk;
fc_sub = fc*ksub_rat^theta;

% Now we see what the final result looks like with the adjusted
% fixed cost.
fix_paras_sub(1) = fc_sub;
[fvalsubf, momssubf, ex_diagssubf, grd] = feval(objfun, guess_p, ...
                                                fix_paras_sub, data_valssub);


% Lets put together the variables we want to output
coldescs = strvcat('R\\&D to assets', 'Profitability', 'Tobin''s Q', ...
                   '$\\bar{j}$');
rowdescs = strvcat('Estimate', 'Experiment');

% Put together the data we want to report
rdm_simdata = [lambda moms(1) moms(14) moms(3) moms(5) moms(4) ex_diags(2)];

rdm_lamdata = [lambda_d momslf(1) momslf(14) momslf(3) momslf(5) momslf(4) ...
           ex_diagslf(2)];
rdm_subsdata = [momssubf(1) momssubf(14) momssubf(3) momssubf(5) momssubf(4) ...
            ex_diagssubf(2)];

svfil = 'ExperVals';
save(svfil, 'rdm_simdata', 'rdm_lamdata', 'rdm_subsdata', '-append');

