function [gmm_val, mom_sim, diags, g, r] = GMMObj_multmod(points, paras, data);
% ------------------------------------------------------------ %
% PURPOSE:  This program is written to evaluate the objective  %
%	    function to be minimized for the GMM approach. The %
%	    computation of this objective function involves    %
%	    simulating the model at the steady state and then  %
%	    comparing the model moments with the actual data.  %
%	    The specific moments to employ and the weighting   %
%	    matrix remain to be settled.		       %
% ------------------------------------------------------------ %
% INPUTS:                                                      %
%     Model points to be estimated                             %
%       theta  - curvature of profit function		       %
%       nu     - inverse elasticity of output                  %
%	rho    - autoregressive term of AR(1) equation	       %
%	sig    - conditional standard deviation of AR(1)       %
%	lambda - jump in profitability from successful r&d     %
%	a      - coefficient in exponential distribution       %
%	b      - convex adjustment cost parameter	       %
%       gamma  - depreciation rate for the knowledge stock     %
%     Precpecified parameters                                  %
%       beta   - discount rate parameter		       %
%	delta  - depreciation rate for capital		       %
%       fc     - fixed operating cost per period               %
%	nFirms - number of firms in simulated sample	       %
%       nYears - number of years (panels) in simulated sample  %
%       p_ind  - index measuring burn-in sample size for firms %
%     Data to be used for GMM objective function               %
%	frac_rd- fraction of firms with positive r&d in data   %
%	moms   - matched moments from the data		       %
%	n      - number of observations in the dataset	       %
%	phi    - covariance matrix for data moments	       %
%                                                              %
% ------------------------------------------------------------ %
% OUTPUTS:						       %
%	gmm_val- value of the GMM minimization function	       %
%	mom_sim- simulated set of moments		       %
%	sbar   - steady state success rate from r&d spending   %
%	sb_sim - simulated success rate from r&d spending      %
%	mu_krd - mean size of firms in r&d sector	       %
%       g      - final value function with r&d spending        %
%							       %
% ------------------------------------------------------------ %
% NOTES:						       %
%    This program computes the GMM minimization value to be    %
% used as an input for the subsequent parameter search.  First %
% this program will be used to just do a search over a bunch   %
% of initial guesses.					       %
% ------------------------------------------------------------ %
% OWNERSHIP:						       %
% Written By: Missaka Warusawitharana			       %
% Date:	04/27/2007					       %
% Last Update: 06/18/2007				       %
%      The program has been modified to handle input/output in %
% a more compact manner.  This is primarily for ensuring ease  %
% of calling with the SimAnneal_RD program.  The one thing to  %
% be careful here is to map the matrices to the correct labels.%
%                                                              %
%      07/19/2007 - The program has been simplified to drop the%
% search for the steady state success rate and the externality %
% from r&d investment in the economy.                          %
%      	   	      	     				       %
%      12/27/2010 - The program is rewritten to have more      %
% in the estimation. We will also focus only on firms that do  %
% r&d in the data.
% ------------------------------------------------------------ %

% Fix the mu parameter as it is currently not identified
mu = -0.3;

% This is somewhat uncomfortable, but we need to assign the points
% to be estimated to their model names.
theta = points(1);
nu = points(2);
rho = points(3);
sig = points(4);
lambda = points(5);
a = points(6);
b = points(7);
gamma = points(8);

% The next thing to do is to match the preset parameters
fc = paras(1);
beta = paras(2);
delta = paras(3);
p_ind = paras(4);
nFirms = paras(5);
nYears = paras(6);
trat_rd = paras(7);
srate = paras(8);

% The final (and tricky) thing to do is to handle the rest of the
% stuff
moms = data{1};
n = data{2};
phi = data{3};
grd = data{4};
sbar = data{5};
rdi = data{6};

% Do a stupid check to make sure things line up ok.
if length(points) ~= 8 | length(paras) ~= 8
   display 'Input points do not correspond to estimated points or preset paras'
   gmm_val = 0;
   mom_sim = 0;
   diags = 0;
   return;
end

% Parameters about the function call to solve for the simulated values
sim_ind = p_ind;

% Now we get in to the meaningful part of the program

% Assign the value to the next loop
ginit = grd;
rdinit = rdi;

% This part of the program will now focus on simulating a full data
% set and generating moments for evaluation.  Basically, we will
% then use these moments as part of a search over the parameters.
% Call the value function to solve for firm value
[g, q, qd, i, r, kr, z, kg, w, ec] = ValFun_MultMod(theta, nu, mu, rho, sig, lambda, a, b, ...
                            beta, delta, gamma, fc, trat_rd, sbar, srate, ginit, rdinit);
                        
% Here we try to catch an error with the value function               
if ec == 1    
    display 'Value function error in Simulation - returning to main'           
    gmm_val = 1e10;
    mom_sim = ones(size(moms));
    diags = [0 0 1 1 1];
    g = 0; g1 = 0;    
    return;
end

% Here we try to catch non-convergence of the value function               
if ec == 2    
    display 'Value function non-convergence in Simulation - returning to main'           
    gmm_val = 1e+8;
    mom_sim = ones(size(moms));
    diags = [sbar sbar 1 1 1];
    g = 0; g1 = 0;    
    return;
end

% This part will pick up problems with infinite capital stocks
if ec == 5
    display 'Value function error with infinite ks - returning to main'
    gmm_val = 1e10;
    mom_sim = ones(size(moms));
    diags = [0 0 1 1 1];
    g = 0; g1 = 0;
    return;
end

md_kg = median(kg);

% Now we can call the simulation program for firms with r&d spending
[zp, kp, qp, ip, rp, dp, pip, pir, sp, yp, sgp, succ, countz] = ...
         SimPanelMult(g, q, i, r, kr, z, kg, w, nFirms, nYears, theta, nu, mu, rho, ...
                  sig, lambda, a, b, trat_rd, sbar, beta, delta, gamma, fc, srate);

% Here we can analyse the results by making a combined set of
% policy function matrices
zm = [zp(sim_ind:end,:)];
km = [kp(sim_ind:end,:)];
im = [ip(sim_ind:end,:)];
qm = [qp(sim_ind:end,:)];
pim = [pip(sim_ind:end,:)];
prm = [pir(sim_ind:end,:)];
dm = [dp(sim_ind:end,:)];
scm = [succ(sim_ind:end,:)];
rpm = [rp(sim_ind:end,:)];
rsm = [rp(sim_ind:end,:)./yp(sim_ind:end,:)];
sgm = [sgp(sim_ind:end,:)];
ym = [yp(sim_ind:end,:)];

% Lets forget about all the stupid stuff and construct the actual
% moments of interest from the simulated data.  

% Do the moments for r&d
fm_rd = mean(vec(rsm));
std_rd = sqrt(mean(vec(rsm.^2)) - fm_rd^2);

% Here we want the auto-regressive term for r&d
% We want to do some error trapping here.
try
    beta_rd = regress(vec(rsm(2:end,:)), adc(vec(rsm(1:end-1,:))));
    rho_rd = beta_rd(2);
catch
    rho_rd = 0;
end

% Pick up the sales to capital ratio
fm_sk = mean(vec(ym));

% Do the moments for q
fm_q = mean(vec(qm));

% Do the moments for income after rd
fm_pr = mean(vec(prm));
std_pr = sqrt(mean(vec(prm.^2)) - fm_pr^2);

% Carry out the regression for income
prdiff = prm - fm_pr;
beta_inc = regress(vec(prdiff(2:end,:)), adc(vec(prdiff(1:end-1,:))));
rho_pr = beta_inc(2);
% beta_inc = regress(vec(prm(2:end,:)), adc(vec(prm(1:end-1,:))));
% rho_pr = beta_inc(2);

% Now we want investment
fm_i = mean(vec(im));  % We will exclude this for now
std_i = sqrt(mean(vec(im.^2)) - fm_i^2);

% Pick up some correlation matrices
co_rdinv = corr(rsm(:), im(:));
co_rdq = corr(rsm(:), qm(:));
co_qi = corr(qm(:), im(:));
co_sglrd = corr(vec(sgm(2:end,:)), vec(rsm(1:end-1,:)));

% Put in an adjustment for NaN problems with correlations
if isnan(co_rdinv)
    co_rdinv = 0;
end
if isnan(co_rdq)
    co_rdq = 0;
end
if isnan(co_qi)
    co_qi = 0;
end
if isnan(co_sglrd)
    co_sglrd = 0;
end

fm_rdk = mean(vec(rp(sim_ind:end,:)));

% Compute some other measures of interest to report back
sb_sim = mean(vec(scm));
mu_krd = mean(vec(kp(sim_ind:end,:)));

% Put everything together into a big matrix
mom_sim = [fm_rd std_rd fm_sk fm_q fm_pr std_pr std_i ...
           rho_pr rho_rd co_rdinv co_rdq co_qi co_sglrd ...
	   fm_rdk mu_krd sb_sim];

% Lets compute some type of a difference measure
mom_diff = mom_sim(1:length(moms)) - moms;
gmm_val = n * mom_diff * inv(n*phi) * mom_diff';

% Compute some other measures of interest to report back
sb_sim = mean(vec(scm))/(1+mean(vec(scm)));
mu_krd = mean(vec(kp(sim_ind:end,:)));

krat(1) = mu_krd / md_kg;
kgmax = max(kg);
kgmin = min(kg);

% Here we do some adjustments for firm sizes being out of whack
if (krat > 2.5 | krat < 0.4)
    simerr = 10;
    gmm_val = gmm_val * 1e3;
else
    simerr = 0;
end    

% Put the diagnostics into one matrix
diags = [sbar sb_sim mu_krd md_kg kgmin kgmax simerr];

% Return the calling function
return;
