% ------------------------------------------------------------ %
% PURPOSE:  This program is written to make some pictures      %
%           using the various moments we calculated from the   %
%           model with parameter values near the optimum.  We  %
%           will also be plotting the GMM value function too.  %
%           This should be a fairly straightforward program.   %
% ------------------------------------------------------------ %

clear all;
close all;

% Load in the data from the file
load ../RDOnly/identvals;

% Now we start to make the pictures
% First select the moments we want to plot
momplot = [1 2 9 5 4];
nplot = length(momplot);

% Figure out the range of values we will be plotting these over
range = 1-epsv:epsv_val:1+epsv;
range_vals(1,:) = points(4)*range;
range_vals(2,:) = points(5)*range;
range_vals(3,:) = points(7)*range;

% Pick up the plotting axis
xmin(1) = points(4)*(1-epsv*6/5);
xmin(2) = points(5)*(1-epsv*6/5);
xmin(3) = points(7)*(1-epsv*6/5);

xmax(1) = points(4)*(1+epsv*6/5);
xmax(2) = points(5)*(1+epsv*6/5);
xmax(3) = points(7)*(1+epsv*6/5);

% Label the various parameters
varlabs = strvcat('\lambda', 'a', '\gamma');

% We also want to name the various moments
momcats = strvcat('Average', 'Standard deviation', 'Autocorrelation');
momvars = strvcat(' R&D to sales', ' profitability', 'Tobin''s q');

% Now we can start by plotting the gmm-value function.  First,
% avoid numerical error problems by smoothing out the gmm function
% values
gmm_val_min = gmm_val_f(1,1);

gmm_val_f = max(gmm_val_min, gmm_val_f);
gmm_val_b = max(gmm_val_min, gmm_val_b);

% Now we need to combine together the data we have. We have to flip
% the order of the values in the backward matrix here.
gmm_val = [flipord(gmm_val_b, 2) gmm_val_f(:,2:end)];

% Next use a loop to combine together the various moments
nmom = length(moms(mompick));
for k = 1:nmom
    mom_sim(:,:,k) = [flipord(mom_sim_b(:,:,k), 2) mom_sim_f(:,2:end,k)];
end

% Now we start plotting
% Specifying the plot styles we use
plotsym = strvcat('b-', 'g-', 'r-');

% Pick up the fontsize
fs = 18;

% Now we want to do the smoothing
np = 3;
for k = 1:3
    [gmm_fval(k, :), cvg(k)] = PolySmooth(gmm_val(k, :), ...
                                          range_vals(k, :),np);
end

% Now we want to find the best ranges for this
scalmax = 1e3;
ymin = floor(min(gmm_fval(:))*scalmax)/scalmax;
ymax = ceil(max(gmm_fval(:))*scalmax)/scalmax;

% We want to set the x-axis ticks for lambda
xticks = 0.2175:0.0025:0.2275;

% Do the plot for GMM function values
for i = 1:3
    svname = strcat('Plots/gmmplot', int2str(i));
    figure;
    plot(range_vals(i, :), (gmm_fval(i, :)), plotsym(i, :));
    axis([xmin(i) xmax(i) ymin ymax]);
    set(gca, 'PlotBoxAspectRatio', [1.5 1 1]);
    xlabel(varlabs(i, :), 'FontSize', fs);
    ylabel('Sum-of-squared differences', 'FontSize', fs);
    set(gca, 'FontSize', fs);
    if i == 1
        set(gca, 'XTick', xticks);
    end
    saveas(gcf, svname, 'epsc');
end

% Now we need to do smooth over the plotted values
mom_fsim = mom_sim;
for j = 1:nplot
    for i = 1:3
        [mom_fsim(i, :, momplot(j)), cvmom(i, j)] = ...
              polysmooth(mom_sim(i, :, momplot(j)), ...
                         range_vals(i, :), np);        
    end
    ym_min(j) = floor(scalmax*min(vec(mom_fsim(:, :, momplot(j)))))/scalmax;
    ym_max(j) = ceil(scalmax*max(vec(mom_fsim(:, :, momplot(j)))))/scalmax;
end

% The next step is to loop over the selected moments

% Increase the fontsize first
fs = 24;

% Make a vector of tick labels
yticks{1} = 0.08:0.0005:0.083;
yticks{2} = 0.027:0.0005:0.029;
yticks{3} = 0.695:0.005:0.72;
yticks{4} = 0.355:0.0025:0.375;
yticks{5} = 3.54:0.01:3.59;

for i = 1:3
    for j = 1:nplot
        svname = strcat('Plots/momplot', int2str(j), '_', ...
                        int2str(i));
        figure;
        plot(range_vals(i, :), mom_fsim(i, :, momplot(j)), ...
             plotsym(i, :));
        axis([xmin(i) xmax(i) ym_min(j) ym_max(j)]);
        set(gca, 'PlotBoxAspectRatio', [1.25 1 1]);
        set(gca, 'FontSize', fs);
        xlabel(varlabs(i, :), 'FontSize', fs);
        if j <= 3
            ylab = strcat(momcats(j, :), ' of', momvars(1, :));
        else
            ylab = strcat(momcats(1, :), ' of', momvars(j-2, :));
        end
        if i == 1
            set(gca, 'XTick', xticks);
        end
        ylabel(ylab, 'FontSize', fs);
        set(gca, 'YTick', yticks{j});
        saveas(gcf, svname, 'epsc');        
    end
end
