function [mpoint, mfval, nacc, nfeval, nobd, ftemp, vmfin, ier, xd, msim, diag, fstor] = ...
	 SimAnneal_RD(objfun, np, guess, paras, data, minmax, rt, eps, ...
                      ns, nt, neps, maxeval, lbound, ubound, c, itemp, vm, nmom)  
% ------------------------------------------------------------ %
% PURPOSE: This program is written to implement the simulated  %
%	   annealing algorithm for a fairly general set of     %
%	   problems.  The best structure for this would be to  %
%	   rely on outside functions which tackle the issue of %
%	   computing the objective function and tackle the     %
%	   transition rules for the parameter space.	       %	  
% ------------------------------------------------------------ %
% INPUTS:                                                      %
%      objfun - Objective function to be minimized             %
%      np     - Number of parameters to be optimized	       %
%      guess  - Initial guess for starting values	       %
%      paras  - Pre-specified parameters in the function       %
%      data   - Input data to pass through to function	       %
%      minmax - Multiplicative factor for min/max choice       %
%      rt     - Cooling rate				       %      
%      eps    - Degree of accuracy for function evaluations    %
%      ns     - Number of evaluations per direction in cycle   %
%      nt     - Number of changes in VM before change of T     %
%      neps   - Number of epsilon changes before finishing     %             
%      maxeval- Maximum number of function evaluations	       %
%      lbound - Lower bound for the parameter space	       %
%      ubound - Upper bound for the parameter space	       %
%      c      - Parameter in adjusting step changes	       %
%      itemp  - Initial temperature                            %
%      vm     - Initial set of changes for parameter space     %
%                                                              %
% OUTPUTS:                                                     %
%      mpoint - Parameter values at the minimum point          %
%      mfval  - Minimized function value                       %
%      nfeval - Number of function evaluations before end      %
%      nobd   -	Number of points outside the bounds	       %
%      ftemp  - Temperature value at the end                   %
%      vmfin  - Final level of step changes in the program     %
%      ier    -	Error code for determining conclusion	       %
%      xd     - Vector of guessed values for parameters	       %
%      msim   - Simulated vector of moments for points	       %
%      diag   -	Vector of diagnostics for tracking code	       %
% ------------------------------------------------------------ %
% NOTES:                                                       %
%      This program has been written with a fair amount of     %
% generality to enable it to tackle many problems using this   %
% technique.  One thing which remains to be settled is the best%
% way of determining when to terminate the program.  Another   %
% tricky problem arises with the selection of how the vector   %
% for searching the parameter space varies.                    %
% ------------------------------------------------------------ %
% AUTHORSHIP:                                                  %
% Written By: Missaka Warusawitharana                          %
% Date: 06/13/2007                                             %
% Last Updates:                                                %  
%      The program has been modified to now store and report   %
% various diagnostic information from the simulation.  This may%
% involve having some large matrices floating around.	       %
% ------------------------------------------------------------ %

% Initialize many of the variables
nobd = zeros(np, 1);
xopt = zeros(np, 1);
fstar = zeros(neps, 1);
ier = 99;
nfeval = 0;
fopt = 0;
nacc = 0;
nacp = zeros(np, 1);

fid = fopen('outp_medeq1.txt', 'w');

% Initialize some of the diagnostic matrices
xd = zeros(maxeval, np);
msim = zeros(maxeval, nmom);
diag = zeros(maxeval, 5);

% Here we do some stupid translating if things are row matrices
% instead of the column matrices required by the program.
if size(guess, 1) == 1
   guess = guess';
end
if size(vm, 1) == 1
   vm = vm';
end
if size(ubound, 1) == 1
   ubound = ubound';
end
if size(lbound, 1) == 1
   lbound = lbound';
end

% Assign some values at the beginning
mpoint = xopt;
mfval = fopt;
temp = itemp;

% Do a few checks to ensure that things will be ok
if itemp < 0
   display 'Initial temperature negative - program failure'
   ier = 1;
   return
end

i = 0;

% Check to see if any of the bounds apply
if max(guess < lbound | guess > ubound) == 1
   display 'Initial point estimates out of bounds - program failure'
   ier = 2;
   return
end

% Evaluate the function to initialize things and adjust for minmax sign
% We will write this as a maximization routine
[fval, moms, ex_diags, grd] = feval(objfun, guess, paras, data);

% We want to repeat this exercise to make sure that our initial points are not that crazy
data{4} = grd;
data{5} = ex_diags(2);
[fval, moms, ex_diags, grd] = feval(objfun, guess, paras, data);

f = minmax * fval;
mopt = moms;
dval = ex_diags;
display 'Minimized GMM criterion equals'
-f
display 'Guessed probabilities'
ex_diags(1:2)
nfeval = nfeval + 1;
fopt = f;
x = guess;
xopt = guess;
fstar(1) = f;

% Store the stuff
xd(nfeval, :) = x';
msim(nfeval, :) = moms;
diag(nfeval, :) = ex_diags;
fstor(nfeval) = f;
grd_opt = grd;
sb_opt = ex_diags(2);

% Here we start our main loop to get to the simulated annealing part
% Initially check using number of function calls
while nfeval <= maxeval
   
   % Here we define some more variables to use in the main loop
   nup = 0;
   nrej = 0;
   nnew = 0;
   ndown = 0;
   lnobd = 0;
   
   % Use for loops to cycle through the parts of the program
   for ql = 1:nt

       for vml = 1:ns

	   for pl = 1:np
	       % Pick up a new guess for one parameter
               xp = x;
	       xp(pl) = xp(pl) + (rand()*2-1)*vm(pl);

	       % Check to ensure guess is within bounds of program
	       % Randomly draw another value if outside the bound
	       if xp(pl) < lbound(pl) | xp(pl) > ubound(pl)
		  xp(pl) = lbound(pl) + rand() * (ubound(pl) - lbound(pl));
		  nobd = nobd + 1;
		  lnobd = lnobd + 1;
	       end

	       % Time to call the function with the point xp
               % First do some adjustments to see if we want to
               % use last solutions for value functions
	       [fval, moms, ex_diags, grd] = feval(objfun, xp, paras, data);
               fp = fval * minmax;
               display 'Minimized gmm criterion equals'
               -fp
	       nfeval = nfeval + 1;

               % Here we reset the seeds to make sure that we get different draws
               rand('state',sum(100*clock));

	       % Store the stuff
	       xd(nfeval, :) = xp';
	       msim(nfeval, :) = moms;
	       diag(nfeval, :) = ex_diags;
               fstor(nfeval) = fp;
               
	       % Here we tackle the acceptance or rejection
               % This somewhat perverse loop is intended to make sure that we
               % dont have problems with guesses for success not lining up correctly.
	       % First accept if we get a higher value
               if fp > fopt
                  [fval, moms, ex_diags, grd] = feval(objfun, xp, paras, data);
                  fp = fval * minmax;
                  data{4} = grd;
                  data{5} = ex_diags(2);
               end
	       if fp > f
		  x = xp;
		  f = fp;
		  nacc = nacc + 1;
		  nacp(pl) = nacp(pl) + 1;
		  nup = nup + 1;
                  data{4} = grd;
                  data{5} = ex_diags(2);
		  
		  % Here do a check to see if this is a new global maxima
		  if fp > fopt
		     xopt = xp;
		     fopt = fp;
                     dval = ex_diags;
		     nnew = nnew + 1;
		     mopt = moms;
		     grd_opt = grd;
		     sb_opt = ex_diags(2);
		  end

	       % Here we use a MC acceptance rule if the point is lower
	       % This depends on the gap between the values and the current 
	       % temperature in a somewhat unusual way.
	       else
	          tval = exp((fp - f)/temp); 
		  rnval = rand();
		  if rnval < tval
		     % Same as regular acceptance	     
		     x = xp;
		     f = fp;
		     nacc = nacc + 1;
		     nacp(pl) = nacp(pl) + 1;
		     ndown = ndown + 1;
                     data{4} = grd;
                     data{5} = ex_diags(2);
		  else
		     nrej = nrej + 1;
		  end
	       end

	       % Now we have finished the accept-reject step.
	       % The program loops back and iterates through each
               % point
           end
           
           % Now we loop back and iterate through the number of steps
       end
       
       % This part of the computation involves reoptimizing 
       ratio = nacp/ns;
       rbig = find(ratio > .6);
       rsmall = find(ratio < .4);
       
       % Adjust VM depending on whether the ratio was big or small
       if ~isempty(rbig)
	  vm(rbig) = vm(rbig).*(1 + c*(ratio(rbig)-.6)/.4);
       end
       if ~isempty(rsmall)
          vm(rsmall) = vm(rsmall)./(1 + c*(.4-ratio(rsmall))/.4);
       end

       % Do a quick check if our vm value is too big
       vmbig = find(vm > ubound - lbound);
       if isempty(vmbig) == 0
           vm(vmbig) = (ubound(vmbig) - lbound(vmbig));
       end
       
       % Reset the acceptance rate in the loop to be zero
       nacp = zeros(np,1);
       
       % Repeat for different iterations before changing temperature
   end
       
   % Here we have to check to see if the termination criterion is
   % met for the previous neps minimum values
   
   fstar(1) = f;
   
   % Here we check for convergence criterion

   if or(((fopt - f) < eps & max(abs(f-fstar)) < eps), (max(vm) < 1e-15))
       
     mpoint = xopt;
     ier = 0;
     ftemp = temp;
     mfval = minmax*fopt;
     vmfin = vm;

     totmov = nup + ndown + nrej;

     fprintf(fid,' intermediate results before next temperature reduction \n');
     fprintf(fid,' current temperature: %g \n', temp);
     fprintf(fid,' min function value: %7g \n', fopt);
     fprintf(fid,' total moves:               %7.0f \n', totmov);
     fprintf(fid,'     downhill:              %7.0f \n',  nup);
     fprintf(fid,'     accepted uphill:       %7.0f \n',  ndown);
     fprintf(fid,'     rejected uphill:       %7.0f \n',  nrej);
     fprintf(fid,' trials out of bounds:      %7.0f \n',  lnobd);
     fprintf(fid,' new minima this temperature: %3.0f \n',  nnew);
     fprintf(fid,' total number of function calls: %7.0f \n',  nfeval);
     fprintf(fid, strcat(' current optimal x: %6.4f %6.4f %6.4f %6.4f', ...
                       ' %6.4f %6.4f %6.4f %6.4f \n'), x);
     fprintf(fid, strcat(' step length (vm): %4.3g %4.3g %4.3g %4.3g', ...
                       ' %4.3g %4.3g %4.3g %4.3g \n'), vm);
     fprintf(fid, strcat(' simulated moments: %5.3f %5.3f %5.3f %5.3f', ...
                       ' %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f \n'), mopt); 		       

     fprintf(fid, 'program converged \n');
     return;

   % If the minimization criteria doesn't work, prepare for another loop     
   else
       
     temp = rt * temp;
     fstar(2:neps) = fstar(1:neps-1);
     f = fopt;
     x = xopt;
     data{4} = grd_opt;
     data{5} = sb_opt;
   
     save res_medeq1;
   end
   
   % Here we write out some output to see what is going on.  This
   % will be helpful for checking to make sure the program works
   % well. 
   
   totmov = nup + ndown + nrej;

   fprintf(fid,' intermediate results before next temperature reduction \n');
   fprintf(fid,' current temperature: %g \n', temp);
   fprintf(fid,' min function value so far: %7g \n', fopt);
   fprintf(fid,' total moves:               %7.0f \n', totmov);
   fprintf(fid,'     downhill:              %7.0f \n',  nup);
   fprintf(fid,'     accepted uphill:       %7.0f \n',  ndown);
   fprintf(fid,'     rejected uphill:       %7.0f \n',  nrej);
   fprintf(fid,' trials out of bounds:      %7.0f \n',  lnobd);
   fprintf(fid,' new minima this temperature: %3.0f \n',  nnew);
   fprintf(fid,' total number of function calls: %7.0f \n',  nfeval);

   fprintf(fid, strcat(' current optimal x: %5.3f %5.3f %5.3f %5.3f', ...
                       ' %5.3f %5.3f %5.3f %5.3f \n'), x);
   fprintf(fid, strcat(' step length (vm): %4.3g %4.3g %4.3g %4.3g', ...
                       ' %4.3g %4.3g %4.3g %4.3g \n'), vm);
   fprintf(fid, strcat(' simulated moments: %5.3f %5.3f %5.3f %5.3f', ...
                       ' %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f \n'), mopt);       
   fprintf(fid, strcat(' diagnostics: %4.3f %4.3f %4.3g', ...
                       ' %4.3g %3.0f \n'), dval);
   i = i + 1;
   fprintf(fid, 'current iteration: %4.0f \n\n', i);

   % End the while loop and go back again
end   

% Here we assign the variables to be returned
mpoint = xopt;
ier = 0;
ftemp = temp;
mfval = minmax*fopt;
vmfin = vm;

end
