function [g, qk, qd, ik, rk, kp, z, kg, w, errcode] = ...
         ValFun_Mult(theta, nu, mu, rho, sig, lambda, a, b, beta, ...
          delta, gamma, fc, trat, sbar, srate, g_init, rd_init); 
% ------------------------------------------------------------ %
% PURPOSE:  This program solves for the optimal policies for   %
%           investment in physical capital and research and    %
%           development of the firm as a function of it's      %
%           current profitability.  The solutions rely on the  %
%           analytical policy functions derived from the first %
%           order conditions for r&d.  The optimal investment  %
%           decisions are obtained using a grid search.        %
% ------------------------------------------------------------ %
% INPUTS:                                                      %
%       theta  - curvature of profit function		       %
%       nu     - inverse elasticity of demand		       %
%       mu     - constant term of AR(1) equation               %
%	rho    - autoregressive term of AR(1) equation	       %
%	sig    - conditional standard deviation of AR(1)       %
%	lambda - jump in profitability from successful r&d     %
%	a      - coefficient in exponential distribution       %
%	b      - convex adjustment cost parameter	       %
%       beta   - discount rate parameter		       %
%	delta  - depreciation rate for capital		       %
%       gamma  - depreciation rate for the knowledge stock     %
%       fc     - fixed operating cost per period               %
%       trat   - tax rate on earnings for firms                %
%       sbar   - aggregate success rate with r&d spending      % 
%       srate  - rate of subsidy on r&d taxes	 	       %
%	g_init - starting matrix for firm values (may be zero) %
%	rd_init- starting matrix for r&d policies     	       %
%                                                              %
% OUTPUTS:                                                     %
%       g      - value of capital stock after shock is known   %
%	qk     - expected value of firm given shock	       %
%	qd     - value difference due to r&d given shock       %
%	ik     - optimal investment policy for the firm	       %
%	rk     - optimal r&d expenditure rate for the firm     %
%       z      - grid of values for profitability in the model %
%       kg     - grid of values for the capital stock	       %
%       w      - weights associated with the grid of values    %
%       errcode- this code is helpful for tracking errors      %
% ------------------------------------------------------------ %
% NOTES:                                                       %
%    This program solves a double state variable model of      %
% optimal investment in r&d and capital, where investment in   %
% r&d can lead to an increase in the future cash flows of the  %
% firm.  This takes a reduced form approach to estimating the  %
% returns from r&d.  The solution to the value function will   %
% serve as an input to a simulation program which constructs   %
% a simulated panel of firm data to use in the subsequent      %
% analysis.  The value function itself is now separated into a %
% component of the knowledge stock of the firm and a component %
% of the value of the operations of the firm.                  %
% ------------------------------------------------------------ %
% OWNERSHIP:                                                   %
% Written By: Missaka W.                                       %
% Date: 03/27/2007                                             %
% Last Update: 04/24/07                                        %
%      This program now incorporates externalities from r&d    %
% spending by other firms and incorporates a scale effect. The %
% interesting question is whether the data would support       %
% positive or negative externalities from r&d expenditures for %
% firm level profits.                                          %
% 06/07/07                                                     %
%      The program now has been modified to force the sample   %
% to just operate on a grid for the capital stock.  This risks %
% leading to overstating the volatility of investment.  The    %
% hope is that this will help speed up the computations.       %
% 12/14/07                                                     %
%      The program has now been modified to incorporate taxes  %
% into the problem.  The tax treatment is somewhat artificial  %
% in that taxes are levied on income before interest and       %
% depreciation.                                                % 
% 12/15/10						       %
%      The program has been modified to allow for multiple     %
% successes in a given period, with geometric distribution.    %
% Rather insanely, we now allow for 10 success levels.         %
% ------------------------------------------------------------ % 
	       

% Initialize some key variables in the analysis
nz = 21;     	  % Start out with 15 grid points for profits
nkd = 50;         % Start out with 40 points on one side of capital
nrd = 301;	  % This specifies the number of r&d grid points.
nk = 2*nkd + 1;   % This specifies the actual grid size for capital
delp = 4;         % Determines the step size for capital change
max_iter = 1000;  % Maximum number of iterations for solution - 500
err_max  = 1e-3;  % Convergence criteria for solution
j = 1;            % Loop counter
errcode = 0;      % Error code to catch problems with interp.

stepr = 0.20/(nz-1); 

% Lets define the decreasing returns to scale parameters
% theta = alpha*(1-nu)/(1-(1-alpha)*(1-nu));
% Now theta is defined directly

% Lets just do the capital grid in a loop
delg = (1-delta)^(1/delp) * ones(nk, 1);
delst = cumprod(1./delg);

% To clarify, this setup has nk grid points for capital with the
% minimum level set such that the capital grid looks nice. Then,
% the future capital values are located on nk*ni grid points and we
% will do a grid search over this.

% Initialize the policy matrices
q = zeros(nk, nz);
i = zeros(nk, nz);
r = zeros(nk, nz);

% Some other matrices
kpi = zeros(nk, nz);
cinv = zeros(nk, nz);
eg = zeros(nk, nz);

% Intermediate matrices
div = zeros(nk, nz);
g_s1 = zeros(nk, nz);
eg_s1 = zeros(nk, nz);
g_s2 = zeros(nk, nz);
eg_s2 = zeros(nk, nz);
g_s3 = zeros(nk, nz);
eg_s3 = zeros(nk, nz);
g_s4 = zeros(nk, nz);
eg_s4 = zeros(nk, nz);
g_s5 = zeros(nk, nz);
eg_s5 = zeros(nk, nz);
g_s6 = zeros(nk, nz);
eg_s6 = zeros(nk, nz);
g_s7 = zeros(nk, nz);
eg_s7 = zeros(nk, nz);
g_s8 = zeros(nk, nz);
eg_s8 = zeros(nk, nz);
g_s9 = zeros(nk, nz);
eg_s9 = zeros(nk, nz);
g_s10= zeros(nk, nz);
eg_s10= zeros(nk, nz);
g_s0 = zeros(nk, nz);
eg_s0 = zeros(nk, nz);
g_new = zeros(nk, nz);

eg_ks0 = zeros(nk, nz);
eg_ks1 = zeros(nk, nz);
eg_ks2 = zeros(nk, nz);
eg_ks3 = zeros(nk, nz);
eg_ks4 = zeros(nk, nz);
eg_ks5 = zeros(nk, nz);
eg_ks6 = zeros(nk, nz);
eg_ks7 = zeros(nk, nz);
eg_ks8 = zeros(nk, nz);
eg_ks9 = zeros(nk, nz);
eg_ks10 = zeros(nk, nz);

% We need to work to get the steady state capital stock at 1
usr = 1/beta - 1 + delta;
z_s = usr/theta;

% Back out the mean profitability levels
mu = log(z_s/(1-trat))*(1-rho) - lambda*sbar/(1-sbar);

% Now we check to see if we keep the capital grid unchanged for
% doing the experiments. If this equals -1, we save the ks value
global fixgrid
if fixgrid == 1
     load muvals;
     mu
elseif fixgrid == -1
     save muvals mu;
end

% Here we determine the grid values by adjusting the mean cash flow
mu_rd = mu + lambda * sbar/(1-sbar);

% Now we need to go and extract the grid values
[z_log, Tz, w] = QuadNorm(mu_rd, rho, sig, nz);

% Compute the steady state value for k
% Here too we will adjust this for the tax rate or not?
z_s = exp((mu + lambda * sbar/(1-sbar))/(1-rho))*(1-trat);
% usr = 1/beta - 1 + delta;

% Lets use a somewhat adhoc adjustment to account for asymmetries
% in the success probabilities and firm sizes for r&d sector.
if lambda == 0
     ks = (1.0*theta*z_s/usr)^(1/(1-theta))
else
     ks = (1.0*theta*z_s/usr)^(1/(1-theta))
end

% We need to have a check for infinite ks as it causes problems
if ks == Inf
     qk = 0;
     qd = 0;
     ik = 0;
     rk = 0;
     kp = 0;
     z  = 0;
     kg = 0;
     errcode = 5;
     return;
end

% Initialize the grid for capital and start with a guess for r&d
kg = delst/median(delst)*ks;
r = 0:stepr:0.20; r = ones(nk,1)*r;

% Lets form a grid for rd investment
maxrd = 1.5;
rdg = 0:maxrd/(nrd-1):maxrd;
rdg = rdg';

% Lets turn this into a big matrix
rdg_cost = rdg*(1 - beta*(1-gamma))*(1-srate-trat);
rd_cost = kg*rdg_cost';
rdg_mat = permute(repmat(rd_cost, [1 1 nz]), [1 3 2]);
p_sg = (1 - exp(-a*rdg));
psg_mat = permute(repmat(p_sg, [1 nk nz]), [2 3 1]);

% Note that these probabilities are constructed using grid values
% not the actual optimal r&d decisions taken by firms.
psg0_mat = 1 - psg_mat;
psg1_mat = (1 - psg_mat).*psg_mat;
psg2_mat = (1 - psg_mat).*psg_mat.^2;
psg3_mat = (1 - psg_mat).*psg_mat.^3;
psg4_mat = (1 - psg_mat).*psg_mat.^4;
psg5_mat = (1 - psg_mat).*psg_mat.^5;
psg6_mat = (1 - psg_mat).*psg_mat.^6;
psg7_mat = (1 - psg_mat).*psg_mat.^7;
psg8_mat = (1 - psg_mat).*psg_mat.^8;
psg9_mat = (1 - psg_mat).*psg_mat.^9;
psg10_mat = 1 - psg0_mat - psg1_mat - psg2_mat - psg3_mat - psg4_mat ...
	   - psg5_mat - psg6_mat - psg7_mat - psg8_mat - psg9_mat;

% Here we adjust for z being in log terms
z = exp(z_log);

% Construct the values of z in the event of succes
z_jump1 = z * exp(lambda*(1-2*sbar)/(1-sbar));
zj1_log = log(z_jump1);

z_jump2 = z * exp(lambda*(2-3*sbar)/(1-sbar));
zj2_log = log(z_jump2);

z_jump3 = z * exp(lambda*(3-4*sbar)/(1-sbar));
zj3_log = log(z_jump3);

z_jump4 = z * exp(lambda*(4-5*sbar)/(1-sbar));
zj4_log = log(z_jump4);

z_jump5 = z * exp(lambda*(5-6*sbar)/(1-sbar));
zj5_log = log(z_jump5);

z_jump6 = z * exp(lambda*(6-7*sbar)/(1-sbar));
zj6_log = log(z_jump6);

z_jump7 = z * exp(lambda*(7-8*sbar)/(1-sbar));
zj7_log = log(z_jump7);

z_jump8 = z * exp(lambda*(8-9*sbar)/(1-sbar));
zj8_log = log(z_jump8);

z_jump9 = z * exp(lambda*(9-10*sbar)/(1-sbar));
zj9_log = log(z_jump9);

z_jump10 = z * exp(lambda*(10-11*sbar)/(1-sbar));
zj10_log = log(z_jump10);

z_jump0 = z * exp(-lambda*sbar/(1-sbar));
zj0_log = log(z_jump0);

% Here we have to do a quick initialization of the starting matrix
if norm(g_init) > 0
    g = g_init;
else
    g = (kg.^theta * median(z)*ones(1,nz))*(1-fc)/beta;
end			 

% Figure out the output matrix
% We need to adjust this for taxes using a gross tax rate
meanz = w'*z/sum(w);
fixcost = (median(kg).^theta)*meanz*fc;
output = ((kg.^theta)*z' - fixcost) * (1-trat) + kg*delta*ones(1,nz)*trat;

% We want to pick up the cost involved in investing.  This is going
% to involve a matrix of values over K and K'.
imat = ones(nk, 1)*kg' - (1-delta) * kg * ones(1, nk);
kmat = kg * ones(1, nk);
ci = imat + b/2*imat.*(imat./kmat);
c = repmat(ci, [1 1 nz]);

% We want to pick up the success matrices
kgscal = (kg*ones(1,nz)).^(1-theta);

if norm(rd_init) > 0
  p_s = (1 - exp(-a*rd_init.*kgscal));
else
  p_s = sbar*ones(nk, nz);
end

ps_exp = repmat(p_s, [1 1 nk]);
ps_mat = permute(ps_exp, [1 3 2]);
ps0_mat = 1 - ps_mat;
ps1_mat = (1 - ps_mat).*ps_mat;
ps2_mat = (1 - ps_mat).*ps_mat.^2;
ps3_mat = (1 - ps_mat).*ps_mat.^3;
ps4_mat = (1 - ps_mat).*ps_mat.^4;
ps5_mat = (1 - ps_mat).*ps_mat.^5;
ps6_mat = (1 - ps_mat).*ps_mat.^6;
ps7_mat = (1 - ps_mat).*ps_mat.^7;
ps8_mat = (1 - ps_mat).*ps_mat.^8;
ps9_mat = (1 - ps_mat).*ps_mat.^9;
ps10_mat = 1 - ps0_mat - ps1_mat - ps2_mat - ps3_mat - ps4_mat ...
          - ps5_mat - ps6_mat - ps7_mat - ps8_mat - ps9_mat;

% ps0_mat = zeros(nk, nk, nz); ps1_mat = 1 - ps0_mat;
% ps2_mat = 0; ps3_mat = 0; ps4_mat = 0; ps5_mat = 0;

% This part constructs an odd matrix to use in expanding the value
% function over future capital stocks. This is unnecessary now.

% Here we start the loop
while j <= max_iter
      
      % First thing to do is to compute the expected value of the firm
      % This is easy to do in the event of failed r&d
      g_s0 = myinterp(zj0_log', z_log', g);
      eg_s0 = beta * g_s0 * Tz;

      % If r&d is successful, the firm will get a new draw for z which
      % is equal to the one given by the transition matrix plus lambda
      % So we want to interpolate v for those values of z
      g_s1 = myinterp(zj1_log', z_log', g);
      eg_s1 = beta * g_s1 * Tz; 

      g_s2 = myinterp(zj2_log', z_log', g);
      eg_s2 = beta * g_s2 * Tz; 

      g_s3 = myinterp(zj3_log', z_log', g);
      eg_s3 = beta * g_s3 * Tz; 

      g_s4 = myinterp(zj4_log', z_log', g);
      eg_s4 = beta * g_s4 * Tz; 

      g_s5 = myinterp(zj5_log', z_log', g);
      eg_s5 = beta * g_s5 * Tz; 

      g_s6 = myinterp(zj6_log', z_log', g);
      eg_s6 = beta * g_s6 * Tz; 

      g_s7 = myinterp(zj7_log', z_log', g);
      eg_s7 = beta * g_s7 * Tz; 

      g_s8 = myinterp(zj8_log', z_log', g);
      eg_s8 = beta * g_s8 * Tz; 

      g_s9 = myinterp(zj9_log', z_log', g);
      eg_s9 = beta * g_s9 * Tz; 

      g_s10 = myinterp(zj10_log', z_log', g);
      eg_s10 = beta * g_s10 * Tz; 
      
      % One of the tricky things here is that the policy decision
      % requires solving a joint pair of equations in R and I.
      % This program now works by using the r from the previous
      % loop.  We will try out two different specifications for the
      % probability of success.  The base case will be exponential.
      
      % Do some juggling to get stuff into desired matrix shapes
      eg_expand_s1= ones(nk, 1) * reshape(eg_s1, [nk*nz 1])';
      eg_expand_s2= ones(nk, 1) * reshape(eg_s2, [nk*nz 1])';
      eg_expand_s3= ones(nk, 1) * reshape(eg_s3, [nk*nz 1])';
      eg_expand_s4= ones(nk, 1) * reshape(eg_s4, [nk*nz 1])';
      eg_expand_s5= ones(nk, 1) * reshape(eg_s5, [nk*nz 1])';
      eg_expand_s6= ones(nk, 1) * reshape(eg_s6, [nk*nz 1])';
      eg_expand_s7= ones(nk, 1) * reshape(eg_s7, [nk*nz 1])';
      eg_expand_s8= ones(nk, 1) * reshape(eg_s8, [nk*nz 1])';
      eg_expand_s9= ones(nk, 1) * reshape(eg_s9, [nk*nz 1])';
      eg_expand_s10= ones(nk, 1) * reshape(eg_s10, [nk*nz 1])';
      eg_expand_s0= ones(nk, 1) * reshape(eg_s0, [nk*nz 1])';

      eg_mat_s1 = reshape(eg_expand_s1, [nk nk nz]);
      eg_mat_s2 = reshape(eg_expand_s2, [nk nk nz]);
      eg_mat_s3 = reshape(eg_expand_s3, [nk nk nz]);
      eg_mat_s4 = reshape(eg_expand_s4, [nk nk nz]);
      eg_mat_s5 = reshape(eg_expand_s5, [nk nk nz]);
      eg_mat_s6 = reshape(eg_expand_s6, [nk nk nz]);
      eg_mat_s7 = reshape(eg_expand_s7, [nk nk nz]);
      eg_mat_s8 = reshape(eg_expand_s8, [nk nk nz]);
      eg_mat_s9 = reshape(eg_expand_s9, [nk nk nz]);
      eg_mat_s10 = reshape(eg_expand_s10, [nk nk nz]);
      eg_mat_s0 = reshape(eg_expand_s0, [nk nk nz]);      

      % Now we can pick up what the firms expected value will be
      eg_mult = ps0_mat.* eg_mat_s0 + ps1_mat.* eg_mat_s1 + ...
      	        ps2_mat.* eg_mat_s2 + ps3_mat.* eg_mat_s3 + ...
		ps4_mat.* eg_mat_s4 + ps5_mat.* eg_mat_s5 + ...
		ps6_mat.* eg_mat_s6 + ps7_mat.* eg_mat_s7 + ...
                ps8_mat.* eg_mat_s8 + ps9_mat.* eg_mat_s9 + ...
		ps10_mat.* eg_mat_s10;

      % Here we solve for the optimization policies
      [max_ing, kp_ind] = max(eg_mult - c, [], 2);
      
      % Now we need to start figuring out what g(K',z') is and also
      % figure out some of the other values at the optimum.
      for ki = 1:nk
        for zi = 1:nz
          kpi = kp_ind(ki,zi);
          eg_ks1(ki,zi) = eg_mat_s1(ki, kpi, zi);
          eg_ks2(ki,zi) = eg_mat_s2(ki, kpi, zi);
          eg_ks3(ki,zi) = eg_mat_s3(ki, kpi, zi);
          eg_ks4(ki,zi) = eg_mat_s4(ki, kpi, zi);
          eg_ks5(ki,zi) = eg_mat_s5(ki, kpi, zi);
          eg_ks6(ki,zi) = eg_mat_s6(ki, kpi, zi);
          eg_ks7(ki,zi) = eg_mat_s7(ki, kpi, zi);
          eg_ks8(ki,zi) = eg_mat_s8(ki, kpi, zi);
          eg_ks9(ki,zi) = eg_mat_s9(ki, kpi, zi);
          eg_ks10(ki,zi) = eg_mat_s10(ki, kpi, zi);
          eg_ks0(ki,zi) = eg_mat_s0(ki, kpi, zi);
          cinv(ki,zi) = ci(ki,kpi);
        end
      end
      
      % R&d expenditures will be given by the following grid search
      eg_ks1_mat = repmat(eg_ks1, [1 1 nrd]);
      eg_ks2_mat = repmat(eg_ks2, [1 1 nrd]);
      eg_ks3_mat = repmat(eg_ks3, [1 1 nrd]);
      eg_ks4_mat = repmat(eg_ks4, [1 1 nrd]);
      eg_ks5_mat = repmat(eg_ks5, [1 1 nrd]);
      eg_ks6_mat = repmat(eg_ks6, [1 1 nrd]);
      eg_ks7_mat = repmat(eg_ks7, [1 1 nrd]);
      eg_ks8_mat = repmat(eg_ks8, [1 1 nrd]);
      eg_ks9_mat = repmat(eg_ks9, [1 1 nrd]);
      eg_ks10_mat = repmat(eg_ks10, [1 1 nrd]);
      eg_ks0_mat = repmat(eg_ks0, [1 1 nrd]);

      eg_rd = psg1_mat.*eg_ks1_mat + psg2_mat.*eg_ks2_mat + ...
              psg3_mat.*eg_ks3_mat + psg4_mat.*eg_ks4_mat + ...
              psg5_mat.*eg_ks5_mat + psg6_mat.*eg_ks6_mat + ...
              psg7_mat.*eg_ks7_mat + psg8_mat.*eg_ks8_mat + ...
              psg9_mat.*eg_ks9_mat + psg10_mat.*eg_ks10_mat + ...
	      psg0_mat.*eg_ks0_mat - rdg_mat;
 
      [max_val, rk_ind] = max(eg_rd, [], 3);
      rk = rdg(squeeze(rk_ind));

      % This part involves computing the probabilities of success
      r = rk.*(kg*ones(1,nz));
      p_s = (1 - exp(-a*rk.*kgscal));
      p_s0 = 1 - p_s;
      p_s1 = (1 - p_s).*p_s;
      p_s2 = (1 - p_s).*p_s.^2;
      p_s3 = (1 - p_s).*p_s.^3;
      p_s4 = (1 - p_s).*p_s.^4;
      p_s5 = (1 - p_s).*p_s.^5;
      p_s6 = (1 - p_s).*p_s.^6;
      p_s7 = (1 - p_s).*p_s.^7;
      p_s8 = (1 - p_s).*p_s.^8;
      p_s9 = (1 - p_s).*p_s.^9;
      p_s10 = 1 - p_s0 - p_s1 - p_s2 - p_s3 - p_s4 - p_s5 - p_s6 - ...
              p_s7 - p_s8 - p_s9;

      ps_exp = repmat(p_s, [1 1 nk]);
      ps_mat = permute(ps_exp, [1 3 2]);
      ps0_mat = 1 - ps_mat;
      ps1_mat = (1 - ps_mat).*ps_mat;
      ps2_mat = (1 - ps_mat).*ps_mat.^2;
      ps3_mat = (1 - ps_mat).*ps_mat.^3;
      ps4_mat = (1 - ps_mat).*ps_mat.^4;
      ps5_mat = (1 - ps_mat).*ps_mat.^5;
      ps6_mat = (1 - ps_mat).*ps_mat.^6;
      ps7_mat = (1 - ps_mat).*ps_mat.^7;
      ps8_mat = (1 - ps_mat).*ps_mat.^8;
      ps9_mat = (1 - ps_mat).*ps_mat.^9;
      ps10_mat = 1 - ps0_mat - ps1_mat - ps2_mat - ps3_mat - ps4_mat ...
      	         - ps5_mat - ps6_mat - ps7_mat - ps8_mat - ps9_mat;

      % Here we want to get the value of the firm given success prob.
      eg_new = eg_ks0.*p_s0 + eg_ks1.*p_s1 + eg_ks2.*p_s2 + ...
               eg_ks3.*p_s3 + eg_ks4.*p_s4 + eg_ks5.*p_s5 + ...
               eg_ks6.*p_s6 + eg_ks7.*p_s7 + eg_ks8.*p_s8 + ...
               eg_ks9.*p_s9 + eg_ks10.*p_s10;

      eg = eg_new;

      % Compute dividends
      div = output - cinv - (r - beta*(1-gamma)*r)*(1-srate-trat);
      
      % Iterate on the value of the firm
      g_new = div + eg;

      if max(max(abs(imag(g_new)))) > 0
            display 'Imag. g'
            qk = zeros(nk,nz);
            qd = zeros(nk,nz);
            ik = zeros(nk,nz);
            rk = zeros(nk,nz);
            kp = ones(nk,nz);
            errcode = 1;
	    return;
      end
      
      % Compute the fit of our solution and iterate
      g_new = max(g_new, 0);
      errg = norm(g - g_new)/ks;
      if abs(errg) <= err_max
            break;
      else
            j = j + 1;
	    g = g_new;
      end

end      
          
% Here we want to make sure that non-convergence doesn't cause us
% problems with the simulations and the estimation.  This will then
% be tackled subsequently in the various GMM objective functions.
if j >= max_iter
      errcode = 2;
end

% Here we pick up some stuff for interest
kprime = squeeze(kg(kp_ind));
qk = eg./kprime;
qd = (eg_s1 - eg_s0)./(kg*ones(1,nz));
inv = kprime - kg*(1-delta)*ones(1,nz);
ik = inv ./(kg*ones(1,nz));
rk = r ./(kg*ones(1,nz));
kp = squeeze(kp_ind);

% Presumably we are done now - Lets display some results
disp('Number of iterations')
j
%disp('Q values from solution')
%q
%disp('Policy functions for investment')
%p = [ik]
%disp('Policy functions for r&d')
%p = [rk]
%disp('Difference in value from r&d success')
%suc = qd
%disp('Value of the capital stock')
%g

return;


     
