function [g, qk, qd, ik, rk, kp, z, kg, w, errcode] = ...
         ValFun_Spill(theta, nu, mu, rho, sig, lambda, ...
          a, b, beta, delta, gamma, fc, trat, sbar, srate, g_init); 
% ------------------------------------------------------------ %
% PURPOSE:  This program solves for the optimal policies for   %
%           investment in physical capital and research and    %
%           development of the firm as a function of it's      %
%           current profitability.  The solutions rely on the  %
%           analytical policy functions derived from the first %
%           order conditions for r&d.  The optimal investment  %
%           decisions are obtained using a grid search.        %
% ------------------------------------------------------------ %
% INPUTS:                                                      %
%       theta  - curvature of profit function		       %
%       nu     - inverse elasticity of demand		       %
%       mu     - constant term of AR(1) equation               %
%	rho    - autoregressive term of AR(1) equation	       %
%	sig    - conditional standard deviation of AR(1)       %
%	lambda - jump in profitability from successful r&d     %
%	a      - coefficient in exponential distribution       %
%	b      - convex adjustment cost parameter	       %
%       beta   - discount rate parameter		       %
%	delta  - depreciation rate for capital		       %
%       gamma  - depreciation rate for the knowledge stock     %
%       fc     - fixed operating cost per period               %
%       trat   - tax rate on earnings for firms                %
%       sbar   - aggregate success rate with r&d spending      % 
%       srate  - rate of subsidy on r&d taxes	 	       %
%	g_init - starting matrix for firm values (may be zero) %
%                                                              %
% OUTPUTS:                                                     %
%       g      - value of capital stock after shock is known   %
%	qk     - expected value of firm given shock	       %
%	qd     - value difference due to r&d given shock       %
%	ik     - optimal investment policy for the firm	       %
%	rk     - optimal r&d expenditure rate for the firm     %
%       z      - grid of values for profitability in the model %
%       kg     - grid of values for the capital stock	       %
%       w      - weights associated with the grid of values    %
%       errcode- this code is helpful for tracking errors      %
% ------------------------------------------------------------ %
% NOTES:                                                       %
%    This program solves a double state variable model of      %
% optimal investment in r&d and capital, where investment in   %
% r&d can lead to an increase in the future cash flows of the  %
% firm.  This takes a reduced form approach to estimating the  %
% returns from r&d.  The solution to the value function will   %
% serve as an input to a simulation program which constructs   %
% a simulated panel of firm data to use in the subsequent      %
% analysis.  The value function itself is now separated into a %
% component of the knowledge stock of the firm and a component %
% of the value of the operations of the firm.                  %
% ------------------------------------------------------------ %
% OWNERSHIP:                                                   %
% Written By: Missaka W.                                       %
% Date: 03/27/2007                                             %
% Last Update: 04/24/07                                        %
%      This program now incorporates externalities from r&d    %
% spending by other firms and incorporates a scale effect. The %
% interesting question is whether the data would support       %
% positive or negative externalities from r&d expenditures for %
% firm level profits.                                          %
% 06/07/07                                                     %
%      The program now has been modified to force the sample   %
% to just operate on a grid for the capital stock.  This risks %
% leading to overstating the volatility of investment.  The    %
% hope is that this will help speed up the computations.       %
% 12/14/07                                                     %
%      The program has now been modified to incorporate taxes  %
% into the problem.  The tax treatment is somewhat artificial  %
% in that taxes are levied on income before interest and       %
% depreciation.                                                % 
% 12/15/10						       %
%      The program now has been changed to incorporate tax     %
% deductibility of depreciation and possible tax subsidies to  %
% r&d investment.
% ------------------------------------------------------------ % 
	       

% Initialize some key variables in the analysis
nz = 21;     	  % Start out with 15 grid points for profits
nkd = 100;        % Start out with 40 points on one side of capital
nk = 2*nkd + 1;   % This specifies the actual grid size for capital
delp = 4;         % Determines the step size for capital change
max_iter = 1000;  % Maximum number of iterations for solution - 500
err_max  = 1e-4;  % Convergence criteria for solution
j = 1;            % Loop counter
errcode = 0;      % Error code to catch problems with interp.

stepr = 0.20/(nz-1); 

% Lets define the decreasing returns to scale parameters
% theta = alpha*(1-nu)/(1-(1-alpha)*(1-nu));
% Now theta is included as a parameter.

% Lets just do the capital grid in a loop
delg = (1-delta)^(1/delp) * ones(nk, 1);
delst = cumprod(1./delg);

% To clarify, this setup has nk grid points for capital with the
% minimum level set such that the capital grid looks nice. Then,
% the future capital values are located on nk*ni grid points and we
% will do a grid search over this.

% Initialize the policy matrices
qk = zeros(nk, nz);
ik = zeros(nk, nz);
rk = zeros(nk, nz);

% Intermediate matrices
div = zeros(nk, nz);
g_s = zeros(nk, nz);
eg_s = zeros(nk, nz);
eg_f = zeros(nk, nz);
g_new = zeros(nk, nz);

eg = zeros(nk, nz);
eg_diff = zeros(nk, nz);
kpi = zeros(nk, nz);
cinv = zeros(nk, nz);

% This part reverse-engineers mean profitability levels
usr = 1/beta - 1 + delta;
z_s = usr/theta;

% Back out the mean profitability level to get ks = 1
mu = log(z_s/(1-trat))*(1-rho) - lambda*sbar;
global fixgrid
if fixgrid == 1
     load muvals;
     mu
elseif fixgrid == -1
     save muvals mu;
end
% mu = mu + .1;

% Here we determine the grid values by adjusting the mean cash flow
mu_rd = mu + lambda * sbar;
% mu_rd = mu_rd + .1;

% Now we need to go and extract the grid values
[z_log, Tz, w] = QuadNorm(mu_rd, rho, sig, nz);

% Compute the steady state value for k
% Here too we will adjust this for the tax rate or not?
z_s = exp((mu + lambda * sbar)/(1-rho))*(1-trat);
% usr = 1/beta - 1 + delta;

% Lets use a somewhat adhoc adjustment to account for asymmetries
% in the success probabilities and firm sizes for r&d sector.
if lambda == 0
     ks = (1.0*theta*z_s/usr)^(1/(1-theta))
else
    %ks = (0.8*theta*z_s/usr)^(1/(1-theta))
     ks = (1.0*theta*z_s/usr)^(1/(1-theta))
end

% Now we check to see if we keep the capital grid unchanged for
% doing the experiments. If this equals -1, we save the ks value

% We need to have a check for infinite ks as it causes problems
if ks == Inf
     qk = 0;
     qd = 0;
     ik = 0;
     rk = 0;
     kp = 0;
     z  = 0;
     kg = 0;
     errcode = 5;
     return;
end

% Initialize the grid for capital and start with a guess for r&d
kg = delst/median(delst)*ks;
r = 0:stepr:0.20; r = ones(nk,1)*r;

% Here we adjust for z being in log terms
z = exp(z_log);

% Construct the values of z in the event of succes
z_jump = z * exp(lambda*(1-sbar));
zj_log = log(z_jump);

z_flop = z * exp(-lambda*sbar);
zf_log = log(z_flop);

% Here we have to do a quick initialization of the starting matrix
if norm(g_init) > 0
    g = g_init;
else
    g = (kg.^theta * median(z)*ones(1,nz))*(1-fc)/beta;
end			 

% Figure out the output matrix
% We need to adjust this for taxes using a gross tax rate
meanz = w'*z/sum(w);
fixcost = (median(kg).^theta)*meanz*fc;
output = ((kg.^theta)*z' - fixcost) * (1-trat) + kg*delta*ones(1,nz)*trat;

% We want to pick up the cost involved in investing.  This is going
% to involve a matrix of values over K and K'.
imat = ones(nk, 1)*kg' - (1-delta) * kg * ones(1, nk);
kmat = kg * ones(1, nk);
ci = imat + b/2*imat.*(imat./kmat);
c = repmat(ci, [1 1 nz]);
eg_diff(:) = 1;
ps_mat = sbar*ones(nk, nk, nz);

% This part constructs an odd matrix to use in expanding the value
% function over future capital stocks. This is unnecessary now.

% Here we start the loop
while j <= max_iter
      
      % First thing to do is to compute the expected value of the firm
      % This is easy to do in the event of failed r&d
      g_f = myinterp(zf_log', z_log', g);
      eg_f = beta * g_f * Tz;

      % If r&d is successful, the firm will get a new draw for z which
      % is equal to the one given by the transition matrix plus lambda
      % So we want to interpolate v for those values of z
      g_s = myinterp(zj_log', z_log', g);
      eg_s = beta * g_s * Tz; 
      % eg_s = myinterp(zj_log', z_log', eg_f);      
      
      % One of the tricky things here is that the policy decision
      % requires solving a joint pair of equations in R and I.
      % This program now works by using the r from the previous
      % loop.  We will try out two different specifications for the
      % probability of success.  The base case will be exponential.
      
      % Do some juggling to get stuff into desired matrix shapes
      eg_expand_s = ones(nk, 1) * reshape(eg_s, [nk*nz 1])';
      eg_expand_f = ones(nk, 1) * reshape(eg_f, [nk*nz 1])';
      eg_mat_s = reshape(eg_expand_s, [nk nk nz]);
      eg_mat_f = reshape(eg_expand_f, [nk nk nz]);      

      % Now we can pick up what the firms expected value will be
      eg_mult = ps_mat.* eg_mat_s + (1-ps_mat).* eg_mat_f;  

      % Here we solve for the optimization policies
      [max_ing, kp_ind] = max(eg_mult - c, [], 2);
      
      % This step involves getting expected future values differenced.
      eg_jump = eg_mat_s - eg_mat_f;
      
      % Now we need to start figuring out what g(K',z') is and also
      % figure out some of the other values at the optimum.
      for ki = 1:nk
        for zi = 1:nz
            % kpi = kp_ind(ki,zi);
          eg_diff(ki,zi) = eg_jump(ki,kp_ind(ki,zi),zi);          
          cinv(ki,zi)    = ci(ki,kp_ind(ki,zi));
        end
      end
      
      % R&d expenditures will be given by the following
      % Modify by theta here if necessary.
      if a < 1000
          r = (kg.^theta*ones(1,nz)).*(log(a) - theta*log(kg*ones(1,nz)) - ...
                                log(1-beta*(1-gamma)) - log(1-srate-trat) + ...
                                log(eg_diff))/a;
          % Make sure to adjust for non-negative values of r;
          r = max(r, 0);
      else
          r = zeros(nk, nz);
      end

      % This part involves computing the probabilities of success
      rk = r./(kg*ones(1,nz));
      p_s = (1 - exp(-a*rk.*((kg*ones(1,nz)).^(1-theta))));
      % p_s = (1 - exp(-a*rk));
      ps_exp = repmat(p_s, [1 1 nk]);
      ps_mat = permute(ps_exp, [1 3 2]);

      % Here we want to get the value of the firm given success prob.
      eg_mult = ps_mat.* eg_mat_s + (1-ps_mat).* eg_mat_f;  

      % This part is about figuring out the expected value of the
      % firm in the next period given choices over I and R.
      for ki = 1:nk
        for zi = 1:nz
          eg(ki,zi)      = eg_mult(ki,kp_ind(ki,zi),zi);
        end
      end

      % Compute dividends
      div = output - cinv - (r - beta*(1-gamma)*r)*(1-srate-trat);
      
      % Iterate on the value of the firm
      g_new = div + eg;

      if max(max(abs(imag(g_new)))) > 0
            display 'Imag. g'
            qk = zeros(nk,nz);
            qd = zeros(nk,nz);
            ik = zeros(nk,nz);
            rk = zeros(nk,nz);
            kp = ones(nk,nz);
            errcode = 1;
	    return;
      end
      
      % Compute the fit of our solution and iterate
      g_new = max(g_new, 0);
      errg = norm(g - g_new)/ks;
      g = g_new;
      if abs(errg) <= err_max
            break;
      else
            j = j + 1;
%           g = max(g_new, 0);
      end

end      
          
% Here we want to make sure that non-convergence doesn't cause us
% problems with the simulations and the estimation.  This will then
% be tackled subsequently in the various GMM objective functions.
if j >= max_iter
      errcode = 2;
end

% Here we pick up some stuff for interest
kprime = squeeze(kg(kp_ind));
qk = eg./kprime;
qd = eg_diff./(kg*ones(1,nz));
inv = kprime - kg*(1-delta)*ones(1,nz);
ik = inv ./(kg*ones(1,nz));
rk = r ./(kg*ones(1,nz));
kp = squeeze(kp_ind);

% Presumably we are done now - Lets display some results
disp('Number of iterations')
j
%disp('Q values from solution')
%q
%disp('Policy functions for investment')
%p = [ik]
%disp('Policy functions for r&d')
%p = [rk]
%disp('Difference in value from r&d success')
%suc = qd
%disp('Value of the capital stock')
%g

return;


     
