function [derv_mean, derv_sp] = DerivObj(data, moments, paras)
% ------------------------------------------------------------ %
% PURPOSE: This function is written to compute the derivative  %
% of the objective function which yields the desired moments   %
% as the solution to a minimization problem.  This objective   %
% function is differentiated with respect to these moments.    %
% In a way, the calculation is simple and requires taking a    %
% fairly careful summation of all the objects of interest.     %
% ------------------------------------------------------------ %
% INPUTS:                                                      %
%    data - Cell of matrices containing simulated data	       %
%    moments - vector of moments at which to compute the deriv.%
%    paras - other parameters which may be of relevance	       %
%                                                              %
% OUTPUTS:						       %
%    derv - Estimated derivative of the objective function     %
% ------------------------------------------------------------ %
% OWNERSHIP:						       %
% Written By: Missaka Warusawitharana			       %
% Date: 07/27/07					       %
% Last Update:						       %
%     06/04/2013  - updated to reflect the latest moment set   %
% ------------------------------------------------------------ %
% NOTES:                                                       %
%   The main thing to note here is that this derivative is of  %
% the objective function which yields as a minimum the moments %
% we are matching with the simulated model.  This is different %
% from the GMM objective function we use to solve for the      %
% model parameters.  The analytical formulae for this is given %
% in a yellow note book.                                       %
% ------------------------------------------------------------ %

% The first thing to do obviously is to organize the input data
rddat = data{1};
qdat = data{2};
pidat = data{3};
idat = data{4};
rdl = data{5};
pil = data{6};
il = data{7}; 
sgdat = data{8};
fsdat = data{9};

% The next thing to do is to construct the forward and lagged data
% matrices 
rdf = rddat;
pif = pidat;
idf = idat;

% Lets assign the input moments so that the code is easier to read
fm_rd = moments(1);
std_rd = moments(2);
fm_fs = moments(3);
fm_q = moments(4);
fm_roa = moments(5);
std_roa = moments(6);
std_i = moments(7);
rho_roa = moments(8);
rho_rd = moments(9);
rho_inv = moments(10);
cord_inv = moments(11);
coinv_q = moments(12);
cosg_lrd = moments(13);

% Lets get the other stuff organize too
fm_inv = mean(idat(:));
std_q = std(qdat(:));
fm_lrd = mean(rdl(~isnan(rdl)));
fm_linv = mean(il(:));
fm_sg = mean(sgdat(~isnan(sgdat)));
std_lrd = std(rdl(~isnan(rdl)));
std_sg = std(sgdat(~isnan(sgdat)));
fm_lroa = mean(pil(:));
nFirms = paras(1);

% The next part is simply a matter of carefully writing out the
% various functions which yields the derivatives of interest
derv(:,1) = - (vec(rdf) - fm_rd) + (vec(rdf.^2) - fm_rd^2 - ...
 	      std_rd^2)*2*fm_rd + (vec(rdf) - fm_rd - ...
              rho_rd*(vec(rdl) - fm_rd))*(1-rho_rd) + ...
	      ((vec(rdf.*idat) - fm_rd*fm_inv)/(std_rd*std_i) - cord_inv) ...
	      * fm_inv/(std_rd*std_i) + ...
	      ((vec(rdl.*sgdat) - fm_rd*fm_sg)/(std_rd*std_sg) - cosg_lrd) ...
	      * fm_sg/(std_rd*std_sg);
derv(:,2) = - (vec(rdf.^2) - fm_rd^2 - std_rd^2)*2*std_rd + ...
	      ((vec(rdf.*idat) - fm_rd*fm_inv)/(std_rd*std_i) - cord_inv) ...
	      .*(vec(rdf.*idat) - fm_rd*fm_inv)/(std_rd^2*std_i)  + ...
	      ((vec(rdl.*sgdat) - fm_rd*fm_sg)/(std_rd*std_sg) - cosg_lrd) ...
	      .*(vec(rdl.*sgdat) - fm_rd*fm_sg)/(std_rd^2*std_sg);
derv(:,3) = - (vec(fsdat) - fm_fs);
derv(:,4) = - (vec(qdat) - fm_q) + ...
	      ((vec(qdat.*idat) - fm_q*fm_inv)/(std_q*std_i) - coinv_q) ...
	      * fm_inv/(std_q*std_i);
derv(:,5) = - (vec(pif) - fm_roa) + (vec(pif.^2) - fm_roa^2 - ...
	      std_roa^2)*2*fm_roa + (vec(pif) - fm_roa - ...
              rho_roa*(vec(pil) - fm_roa))*(1-rho_roa);
derv(:,6) = - (vec(pif.^2) - fm_roa^2 - std_roa^2)*2*std_roa;
derv(:,7) = - (vec(idat.^2) - fm_inv^2 - std_i^2)*2*std_i + ...
	      ((vec(rdf.*idat) - fm_rd*fm_inv)/(std_rd*std_i) - cord_inv) ...
	      .*(vec(rdf.*idat) - fm_rd*fm_inv)/(std_rd*std_i^2)  + ...
	      ((vec(idat.*qdat) - fm_inv*fm_q)/(std_i*std_q) - coinv_q) ...
	      .*(vec(idat.*qdat) - fm_inv*fm_q)/(std_i^2*std_q);
derv(:,8) = - ((vec(pif) - fm_roa) - rho_roa*(vec(pil) - ...
                   fm_roa)).*(vec(pil) - fm_roa);
derv(:,9) = - ((vec(rdf) - fm_rd) - rho_rd*(vec(rdl) - ...
                   fm_rd)).*(vec(rdl) - fm_rd);
derv(:,10) = - ((vec(idf) - fm_inv) - rho_inv*(vec(il) - ...
                   fm_inv)).*(vec(il) - fm_inv);
derv(:,11) = - (vec(rdf.*idat) - fm_rd*fm_inv)/(std_rd*std_i) - cord_inv;
derv(:,12) = - (vec(idat.*qdat) - fm_q*fm_inv)/(std_q*std_i) - coinv_q;
derv(:,13) = - (vec(rdl.*sgdat) - fm_lrd*fm_sg)/(std_lrd*std_sg) - cosg_lrd;

% Lets adjust the derivative matrix for its length
dp = 1 - (derv == 0);

% Compute the mean of the first derivative.
derv_mean = sum(derv)./sum(dp);

% Take differences
derv_diff = derv - dp.*(ones(length(derv),1)*derv_mean);

% Compute the sum of squares
derv_sq = (derv'*derv)./(ones(size(derv,2),1)*sum(dp));
derv_sp = (derv_diff'*derv_diff)./(dp'*dp);
derv_sp(isnan(derv_sp)) = 0;

% Lets see about getting the cross product in a different way;

% End the program
return;
