function [zp, kp, qp, ip, rp, dp, pip, pir, sp, yp, sgp, fsp, succ, countz] = ...
         SimPanel(v, q, i, r, kr, z, kg, w, nFirms, nYears, theta, nu, mu, rho, ...
                  sigma, lambda, a, b, trat, sbar, beta, delta, gamma, fc, srate);
% ------------------------------------------------------------ %
% PURPOSE:  This program constructs a simulated sample of      %
%           firms given their optimal policies for investment  %
%           in physical capital and r&d as a function of its   %
%           current profitability.  The solutions rely on the  %
%           analytical policy functions derived from the first %
%           order conditions.  The simulations then store the  %
%           values of profitability, valuation, investment     %
%	    and r&d for analysis in a panel data set.  The     %
%	    ultimate objective is to use this to obtain a set  %
%           of simulated method of moments estimates of the    %
%	    model parameters.  Sounds fun!	               %
% ------------------------------------------------------------ %
% INPUTS:                                                      %
%       v      - value of the firm                             %
%	q      - expected future value of the prior - Tobins q %
%	i      - optimal investment policy for capital         %
%	r      - r&d investment scaled by capital              %
%	z      - grid of values for profitability              %
%	w      - initial weights to attribute to profitability %
%	kg     - grid of values for the capital stock	       %
%	nFirms - number of firms in simulated sample	       %
%       nYears - number of years (panels) in simulated sample  %
%       theta  - curvature of profit function	       	       %
%       nu     - inverse elasticity term in the demand function%
%       mu     - constant term of AR(1) equation               %
%	rho    - autoregressive term of AR(1) equation	       %
%	sig    - conditional standard deviation of AR(1)       %
%	lambda - jump in profitability from successful r&d     %
%	a      - coefficient in exponential distribution       %
%	b      - convex adjustment cost parameter	       %
%       trat   - tax rate on earnings for firms                %
%       sbar   - steady state rate of success from r&d         %
%       beta   - discount rate in the model                    % 
%       delta  - capital depreciation rate                     %
%       gamma  - knowledge stock depreciation rate             %
%       fc     - per period operating cost of the firm         %
%       srate  - tax subsidy given to r&d investment           %
%                                                              %
% OUTPUTS:                                                     %
%       zp     - profitability of firms after shock (panel)    %
%       kp     - capital stock of firms (panel)                %
%	qp     - expected value of firms given shock (panel)   %
%	ip     - investment policy for the firm	(panel)        %
%	rp     - r&d expenditure rate for the firm (panel)     %
%	dp     - dividend payouts for the firm (panel)         %
%	pip    - operating profits for the firm (panel)        %
%	sp     - stock of knowledge for the firm (panel)       %
%	yp     - operating revenue of the firm (panel)	       %
%	sgp    - sales growth of the firm (panel)	       %
%	fsp    - fixed costs of the firm (panel)	       %
%       succ   - count of firms with successful r&d (panel)    %
%	countz - count of values of z outside the grid	       %
% ------------------------------------------------------------ %
% NOTES:                                                       %
%    This program uses the solution of a dual state variable   %
% model of optimal investment in r&d and capital, where r&d    %
% spending leads to an increase in the future cash flow of the %
% firm.  This takes a reduced form approach to estimating the  %
% returns from r&d.  The simulations rely on the solutions to  %
% the value function given in ValFun.m to construct a panel of %
% firms. The method employed in this analysis relies on spline %
% interpolation of the policies and values of the firms to     %
% obtain the necessary values in between the grid points. This %
% iteration of the program also accounts for spillovers from   %
% successful aggregate r&d expenditures.  One this to note is  %
% here we also allow for some depreciation of the knowledge    %
% stock which impacts the current value of the firm.           %
% ------------------------------------------------------------ %
% OWNERSHIP:                                                   %
% Written By: Missaka W.                                       %
% Date: 04/25/2007                                             %
% Last Update: 06/08/07                                        %
%      This version of the model now forces firms to operate   %
% on a grid of values for capital. This will essentially force %
% companies to have less capital variability and give us a     %
% different way of constructing the simulated sample.  We will %
% have to careful with how to tackle the transition for firm   %
% sizes.
% ------------------------------------------------------------ % 

% Initialize some matrices to start things off
zp = zeros(nYears, nFirms);
kp = zeros(nYears, nFirms);
qp = zeros(nYears, nFirms);
ip = zeros(nYears, nFirms);
rp = zeros(nYears, nFirms);
dp = zeros(nYears, nFirms);
sp = zeros(nYears, nFirms);
yp = zeros(nYears, nFirms);
pip = zeros(nYears, nFirms);
kind = zeros(nYears, nFirms);
int_mat = zeros(3, nFirms);

% Count #num z below and above grid - asymmetries due to impact of r&d
countz = zeros(nYears, 2);

% Define the curvature parameter
% theta = alpha*(1-nu)/(1-(1-alpha)*(1-nu));
% This is now directly estimated.

% Initialize random seeds to ensure replictability
rand('state', 200);
randn('state', 600);

% Initialize a set of z values by using the unconditional weights
cum_w = cumsum(w)/sum(w);
w_init = rand(1, nFirms);
w_indx = zeros(1, nFirms);

% Get the necessary indices by looping over the firms
for k = 1:nFirms
    w_indx(k) = min(find(w_init(k) <= cum_w));
end

% Map these numbers in to our grid for z
% z_c represents the carrying forward values for z
z_c = z(w_indx)';

% Define the log value of z
z_log = log(z);

% Now we need to initialize the capital stocks 
% Pick some numbers for initial distribution capital stocks
% This is now modified to correspond to capital indices
nk = length(kg);
kim = floor((nk+1)/2);
ki_c = kim + round((kim-1)*(rand(1, nFirms)*2-1));
k_c = kg(ki_c,:)';

% Next thing to construct is a set of values for the knowledge
% stock.  I guess we need to do this simply.
s_c = rand(1, nFirms)*0.2.*k_c;

% Pick up the fixed cost
fixcost = (median(kg).^theta)*median(z)*fc;

% Draw a large number of shocks for the transition matrix for z
% Emphasize here that we are using the conditional value for sigma.
shock_z = randn(nYears, nFirms) * sigma;

% Pick up the success probabilities
draw_succ  = rand(nYears, nFirms);

% Initialize a guess for sales;
sales = ((z_c.*k_c.^(theta))/nu*(1-theta));

% I think we have everything needed to start looping over indices
% Just remember to get the indices properly lined up

for j = 1:nYears
    
    % Time to get the optimal policies for investment and r&d using
    % our optimal policy functions.  Then we will compute the
    % dividend payout of the firm, which may be redundant.  The
    % interpolation method of choice is the spline.
    
    % This interpolation involves a tricky expansion of the indices
    % for optimal decisions into a bigger matrix, which will
    % hopefully then provide the necessary information for using a
    % one-dimensional interpolation routine.  
    zc_log = log(z_c);

    % We need to error trap the interpolation - question is what to 
    % do if an error occurs.  We will set to zero for now.
    for ind = 1:nk
      i_ind = (ki_c == ind);
      zr = zc_log(i_ind);
      try
         mat = [q(ind,:)' r(ind,:)' kr(ind,:)'];
         int_mat(:, i_ind) = interp1q(z_log, mat, zr')';
      catch
	 display('zr matrix')
	 zr
	 display('z_log matrix')
	 z_log
	 display('policy matrix')
	 mat
	 int_mat(:, i_ind) = 1;
      end    
    end
    
    ki_c = round(int_mat(3,:));
    ip(j, :) = kg(ki_c)'./k_c - 1 + delta;
    rfp = max(int_mat(2,:), 0);
    rp(j, :) = rfp - s_c*(1-gamma)./k_c;
    
    % Now we want to interpolation firm values and store K, z values.
    qp(j, :) = int_mat(1,:) + beta*rfp*(1-gamma)*(1-srate-trat)./(1-delta+ip(j,:));
    zp(j, :) = z_c;
    kp(j, :) = k_c;
    sp(j, :) = rfp;
    
    % Lets pick up lagged sales
    lsales = sales;

    % Lets store the profit and sales measures;
    outp = (z_c.*k_c.^theta - fixcost);
    sales = ((z_c.*k_c.^(theta))/nu*(1-theta));
    
    % The tricky thing here is that we want to adjust dividends for
    % taxes but not adjust the measure of profits in the simulation.
    dp(j, :) = outp*(1-trat) + k_c*delta*trat - ...
    	       (ip(j,:) + .5*b*ip(j,:).^2 + rp(j,:)*(1-trat)).*k_c;

    % Now we compute the per period operating profit before r&d
    pip(j, :) = (outp)./k_c;				  
    pir(j, :) = pip(j, :) - rp(j, :);  

    % Finally we want to pick up the sales and sales growth values
    yp(j, :) = sales./k_c;
    sgp(j, :) = sales./lsales - 1;
    fsp(j, :) = fixcost./sales;
    
    % Next thing is to compute the capital stock in the next period
    % Make sure that the capital stock stays within the bounds
    k_c = kg(ki_c)';
    
    % Now we want to ensure that we carry forward the knowledge
    % stock of the firm.  Make sure to adjust for scaling by capital.
    s_c = rfp .* kp(j,:);
    
    % One sublety here is that we need to draw these shocks due to
    % the fact that the transition for profitability in this model
    % involves both the occurrence of r&d as well as shocks to
    % z. This might be a bit tricky.

    % As a initial step lets see which firms managed to do some
    % successful r&d in the model. This could be a useful variable
    % to keep track of.
    prob_succ  = 1 - exp(-a*rfp.*(kp(j,:).^(1-theta)));
    % prob_succ  = 1 - exp(-a*rfp);
    succ_ind   = (prob_succ >= draw_succ(j, :));
    succ(j, :) = succ_ind;

    % Change the value of the profitabilities to log values to get
    % the shocks taken care of.  Here we also account for negative
    % externalities from other firm's r&d spending.
    log_z = log(z_c);
    log_zprime = mu + log_z*rho + shock_z(j,:);
    
    % Make the adjustment for cash flow shocks due to r&d success
    log_zprimerd = log_zprime + lambda*succ_ind;
    
    % Translate back to exponential values and save to carrying
    % value and adust for z to take values inside the grid to
    % ensure that nothing mad happens with extrapolating investment
    % and r&d. 
    z_c = exp(log_zprimerd);
    countz(j, 1) = length(find(z_c < z(1)));
    countz(j, 2) = length(find(z_c > z(end)));
    z_c = max(min(z_c, z(end)), z(1));

    % Hmm... this loop might be done
end

% Return to calling program
return;

    
    
