% This program is written to compute the asymptotic variance
% covariance matrix for the baseline model.  This computation will
% closely follow Toni's approach and use the cross product of the J
% matrix as well as numerical derivates computed as of the data
% moments. One of the things which need to be resolved about this
% computation is which set of moments to use to calculate the
% derivative of Jm with respect to the parameters.  I will use the
% values at the data moments, but this is definitely worth testing.

clear;
% Time to add all the paths we need
addpath ../Model;
addpath ../Simul;
addpath ../Quad;
addpath ../Data;
addpath ../basic;

% Obviously the first thing to do is to read in the optimum
% parameters.
theta = 0.4400;
nu = 0.3429;
rho = 0.5746; 
sig = 0.5440;
lambda = 0.1952;
a = 3.8564;
b = 0.7715;
gamma = 0.5157;
fc = 0.3902;
sbar = 0.912;

% We are also going to need the actual data to compute the
% derivates we are interested in.  This is a bit tricky though.
% Too many questions which need to be resolved.

% Define the input and output files for us to read and write
infile = 'softwdata.txt';

% Time to read in the data
[gvkey, yeara, siz, rds, lrds, salek, sgrow, q, roa, lroa, invk, linvk] = ...
	textread(infile, '%f%f%f%f%f%f%f%f%f%f%f%f', 'emptyvalue', NaN, ...
			 'delimiter', '\t', 'headerlines', 1);

% Lets pick up the cross product matrix of Jm, Jm'
load jminnerp_softw;

% We also need to define epsilon for the purpose of the numerical
% derivative. Some care may be needed here as well.
epsval = 2e-3;

% Now one question is exactly which points at which you compute the
% derivative.  It would seem fairly obvious that you do this with
% respect to the actual parameters
readopt = 1;
if readopt == 1
    load xop;
    points = xopt';
else
    points = [theta rho sig lambda a b gamma fc];
end
np = length(points);
epsv = epsval*points;

% Lets also specify the other parameters which go into this problem
beta = 1/1.04;
delta = 0.199;
srate = 0.025;

% These parameters influence the simulation exercise.
nFirms = 509; 
nYears = 100;  
sim_ind = 51;

% Pick up the tax parameters
trat_rd = 0.397;

% We are also going to need the data based moments;
load ../Data/moments_softw;

% Adjust the matrices to the ones we've picked out
mompick = [1 2 4 5 6 7 8 9 10 11 12 15 16];
moms = moms(mompick);
phi = phi(mompick, mompick);

% Pick up the saved file
load gmat;
ginit = g; clear g;
% clear ginit; ginit = 0;

% Well, the first thing to do is to recompute the simulated moments
% at the optimum points.  Lets do this manually partly as a way to
% test what we are doing.

% Describe the moments we are matching to the data
momcats = strvcat('Averages', 'Standard deviations', ['Autocorrelations ' ...
                    ''], 'Correlations between');
momvars = strvcat('R\\&D to sales', 'fixed costs', 'profitability', ...
                  'Tobin''s q', 'investment', 'sales growth');

%momdescs = strvcat('Average of R\\&D to sales', ['Standard deviation ' ...
%                    'of R\\&D to sales'], ['Average of sales to ' ...
%                    'capital '], 'Average of Tobin''s q', ...
%                    'Average of profitability', ['Standard deviation ' ...
%                    'of profitability'], ['Standard deviation of ' ...
%                    'investment'], 'Autocorrelation of profitability', ...
%                    'Autocorrelation of R\\&D to sales', ['Correlation ' ...
%                    'between R\\&D to sales and investment'], ...
%                    ['Correlation between R\\&D to sales and Tobin''s ' ...
%                    'q'], ['Correlation between investment and ' ...
%                    'Tobin''s q'], ['Correlation between sales growth ' ...
%                    'and lagged R\\&D to sales']);
nm = length(moms);
ncats = size(momcats, 1);

% Lets go a bit nuts and start calling these things as a function
[derv_mean, derv_meand, derv_sq, derv_sqd, g, mom_sim, jbar, ...
    prof_in, intrat] = ...
    DerivFun(points, beta, delta, fc, sbar, trat_rd, srate, ...
             ginit, nFirms, nYears, sim_ind, moms, 1);
ginit = g;

% This part involves working with the j-statistics
mom_diff = moms - mom_sim;
jstat = 8/(1+8) * n^2 * mom_diff * inv(n*phi) * mom_diff';
psistat = n^2 * mom_diff * inv(n*phi) * mom_diff';
pval = 1 - chi2cdf(jstat, nm - np);

% Now we try and do the computation of the first order derivatives
% with respect to changes in the parameters
for i = 1:np
    point_inp = points;
    point_inp(i) = points(i) + epsv(i);

    % Well, now we go and get the derivative matrix again
    [derv_mean_up, derv_meand_up, derv_squp, derv_sqdup] = ...
        DerivFun(point_inp, beta, delta, fc, sbar, trat_rd, srate, ...
             ginit, nFirms, nYears, sim_ind, moms, 0);
    
    % Time to repeat with a low value
    point_inp(i) = points(i) - epsv(i);
    [derv_mean_dw, derv_meand_dw, derv_sqdw, derv_sqddw] = ...
        DerivFun(point_inp, beta, delta, fc, sbar, trat_rd, srate, ...
             ginit, nFirms, nYears, sim_ind, moms, 0);
    
    % Well, now we can write down the derivative function
    Q2d_bt(i,:) = (derv_meand_up - derv_meand_dw)/(2*epsv(i));
    Q2_bt(i,:) = (derv_mean_up - derv_mean_dw)/(2*epsv(i)); 

end    

% Here we start wondering about the correct way of getting the
% covariance matrix
toni_avar1 = (1+1/8)*inv(Q2d_bt * inv(jcross) * Q2d_bt');
% toni_avar2 = (1+1/8)*inv(Q2_bt * inv(derv_sq) * Q2_bt');
gmr_avar = (1+1/8)*inv(Q2d_bt * inv(derv_sqd) * Q2d_bt');

% This is the trickiest part of the entire exercise.  We need to
% compute the asymptotic standard errors of the parameter
% estimates.  Lets follow the standard method and hope this is
% right.
stderr = sqrt(diag(toni_avar1/n));
stderr2 = sqrt(diag(gmr_avar/n));

% We also want to figure out the corresponding permanent increase
% in profitability assuming capital fixed
perm_j = (exp(lambda)-1)/(1-rho*beta)*(1-beta);

% I guess the next thing to do is to make some tables - hmm... iffy
% Lets start with the moments tables
momtab = 'softwmoms.txt';
ptab = 'softwparas.txt';
econtab = 'softwecon.txt';

% Lets write a single file with the data we want to report
allsav = 'softwdata.mat';

% Open the file and start writing stuff - 3 column style for moments
fid = fopen(momtab, 'w');

% Start writing the table - stick to a basic format for now
fprintf(fid, '\\begin{tabular}{p{.3cm}lcc} \\\\ \n');
fprintf(fid, '\\hline \n');
fprintf(fid, '\\multicolumn{2}{l}{Moments} & Data & Model \\\\ \n');
fprintf(fid, '\\hline \n');

% Loop over the moments and write them down
for k = 1:ncats
  fprintf(fid, strcat('\\multicolumn{4}{l}{', momcats(k, :), ...
                      '} \\\\ \n'));
  switch k
   case 1
      fprintf(fid, strcat('& ', momvars(1, :), [' & %4.3f & %4.3f ' ...
                            '\\\\ \n']), [moms(1) mom_sim(1)]);
      fprintf(fid, strcat('& ', momvars(2, :), [' & %4.3f & %4.3f ' ...
                            '\\\\ \n']), [moms(3) mom_sim(3)]);
      fprintf(fid, strcat('& ', momvars(3, :), [' & %4.3f & %4.3f ' ...
                            '\\\\ \n']), [moms(5) mom_sim(5)]);
      fprintf(fid, strcat('& ', momvars(4, :), [' & %4.3f & %4.3f ' ...
                            '\\\\ \n']), [moms(4) mom_sim(4)]);
   case 2
      fprintf(fid, strcat('& ', momvars(1, :), [' & %4.3f & %4.3f ' ...
                            '\\\\ \n']), [moms(2) mom_sim(2)]);
      fprintf(fid, strcat('& ', momvars(3, :), [' & %4.3f & %4.3f ' ...
                            '\\\\ \n']), [moms(6) mom_sim(6)]);
      fprintf(fid, strcat('& ', momvars(5, :), [' & %4.3f & %4.3f ' ...
                            '\\\\ \n']), [moms(7) mom_sim(7)]);
   case 3
      fprintf(fid, strcat('& ', momvars(1, :), [' & %4.3f & %4.3f ' ...
                            '\\\\ \n']), [moms(9) mom_sim(9)]);
      fprintf(fid, strcat('& ', momvars(3, :), [' & %4.3f & %4.3f ' ...
                            '\\\\ \n']), [moms(8) mom_sim(8)]);
      fprintf(fid, strcat('& ', momvars(5, :), [' & %4.3f & %4.3f ' ...
                            '\\\\ \n']), [moms(10) mom_sim(10)]);      
   case 4       
      fprintf(fid, strcat('& ', momvars(1, :), ' and %s', ...
                          momvars(5, :), ' & %4.3f & %4.3f \\\\ \n'), ...
                          '', [moms(11) mom_sim(11)]);
      fprintf(fid, strcat('& ', momvars(5, :), ' and %s', ...
                          momvars(4, :), ' & %4.3f & %4.3f \\\\ \n'), ...
                          '', [moms(12) mom_sim(12)]);
      fprintf(fid, strcat('&lagged %s', momvars(1, :), ' and %s', ...
                          momvars(6, :), ' & %4.3f & %4.3f \\\\ \n'), ...
                          '', '', moms(13), mom_sim(13));
  end
    
end    

% Finish up the table
% fprintf(fid, '\\\\ \n');
% fprintf(fid, '\\emph{J}-statistic & & %4.3f \\\\ \n', jstat);
% fprintf(fid, '\\emph{p}-value & & %4.3f \\\\ \n', pval);
fprintf(fid, '\\hline \n');
fprintf(fid, '\\end{tabular} \n');
fclose(fid)

% Now we want to make the second table. Lets stick to Toni's own
% format for horizontal parameters.
fid = fopen(ptab, 'w');
fprintf(fid, '\\begin{tabular}{l*{9}{c}} \\\\ \n ');
fprintf(fid, '\\hline \n');
fprintf(fid, ['Parameter & $\\theta$ & $\\rho$ & $\\sigma$ ' ...
              '& $\\lambda$ & $a$ & $b$ & $\\gamma$ & $\\fc$ & $\\hat{\\Psi}$ \\\\ \n']);
fprintf(fid, '\\hline \n');
fprintf(fid, ['Estimate & %4.3f & %4.3f & %4.3f & %4.3f & %4.3f & %4.3f ' ...
              '& %4.3f & %4.3f & %5.0f \\\\ \n'], [points psistat]);
fprintf(fid, ['Std. err. & (%4.3f) & (%4.3f) & (%4.3f) & (%4.3f) ' ...
              '& (%4.3f) & (%4.3f) & (%4.3f) & (%4.3f) & \\\\ \n'], ...
        stderr);
fprintf(fid, '\\hline \n');
fprintf(fid, '\\end{tabular} \n');
fclose(fid)

% Lets write the summary file to tabulate
save(allsav, 'points', 'stderr', 'moms', 'mom_sim', 'psistat', 'jbar');

return;
